//----------------------------------------------------------------------------------------------------
// SiON Preset voice data
//  Copyright (c) 2008 keim All rights reserved.
//  Distributed under BSD-style license (see org.si.license.txt).
//
// *** REMARKS ***
//  Preset voices set [valsound] is created by Takeshi Abo, and free licensed.
//  Copyright (c) 2009 VAL-SOUND Takeshi Abo ("阿保剛") All rights reserved.
//  url ; VAL-SOUND (http://www.valsound.com/)
//----------------------------------------------------------------------------------------------------

package org.si.sion.utils {
    import org.si.sion.SiONVoice;
    
    
    /** Preset voice data. 457 voices are available now (default:11, valsound:258, GM:128, GMdrum:60).
@example Create new instance and access voices by key.
<listing version="3.0">
var voices:SiONPresetVoice = new SiONPresetVoice(); // create new instance.
var voice:SiONVoice = voices["valsound.piano1"];    // access voice by key
</listing>
@example Or you can access Array list by categoly key.
<listing version="3.0">
var voices:SiONPresetVoice = new SiONPresetVoice(); // create new instance.
var bassVoiceList:Array = voices["valsound.bass"];  // access voice list by categoly key
var bass10:SiONVoice = bassVoiceList[10];           // access voice by index
</listing>
@example Or you can access Array list by categolies property.
<listing version="3.0">
var voices:SiONPresetVoice = new SiONPresetVoice(); // create new instance.
var voiceList:Array = voices.categolies[2];         // access voice list by categoly index
trace(voiceList["name"]);                           // outputs "valsound.bell"
var bell2:SiONVoice = voiceList[2];                 // access voice by index
</listing>
     */
    public dynamic class SiONPresetVoice {
    // constants
    //--------------------------------------------------
        /** Flag to include default voices. */
        public const INCLUDE_DEFAULT:int = 1;
        /** Flag to include voices set [valsound] (http://www.valsound.com/). */
        public const INCLUDE_VALSOUND:int = 2;
        /** Flag to include General MIDI voices. */
        public const INCLUDE_MIDI:int = 4;
        /** Flag to include General MIDI drum set voices. */
        public const INCLUDE_MIDIDRUM:int = 8;
        /** Flag to include all voices. */
        public const INCLUDE_ALL:int = 0xffff;
        
        
        
        
    // valiables
    //--------------------------------------------------
        /** categoly list. */
        public var categolies:Array;
        
        
        
        
    // constructor
    //--------------------------------------------------
        /** constructor.
         *  @param includeFlag Flag to select which voices are included.
         */
        function SiONPresetVoice(includeFlag:int=0xffff) {
            categolies = [];
            
            if (includeFlag & INCLUDE_DEFAULT) {
                _categoly("default");
                _new("sine",      "Sine wave",          new SiONVoice(5,0));
                _new("saw",       "Saw wave",           new SiONVoice(5,1));
                _new("triangle8", "8bit triangle wave", new SiONVoice(5,3));
                _new("triangle",  "Triangle wave",      new SiONVoice(5,4));
                _new("square",    "Square wave",        new SiONVoice(5,5));
                _new("noise",     "White noise",        new SiONVoice(5,6));
                _new("snoise",    "93bit noise",        new SiONVoice(5,25));
                _new("konami",    "Wave table sample",  new SiONVoice(5,7));
                _new("ma1",       "MA3 wave sample",    new SiONVoice(5,33));
                _new("beep",      "Pulse wave sample",  new SiONVoice(5,81));
                _new("ramp",      "Ramp wave sample",   new SiONVoice(5,160));
                
                // 11 default voices
            }

            if (includeFlag & INCLUDE_VALSOUND) {
                _categoly("valsound.bass");
                _OPN("valsound.bass1",  "Analog Bass #2+FBsynth", 6,7, 31,00,00,12,01,18,1,01,0,0, 31,00,01,12,01,04,1,02,0,0, 31,00,00,09,00,03,0,01,7,0, 31,00,00,09,00,03,0,01,3,0);
                _OPN("valsound.bass2",  "Analog Bass",            5,6, 31,00,00,00,00,41,0,01,0,0, 20,00,00,10,00,00,0,01,1,0, 24,00,00,08,00,00,0,01,2,0, 20,00,00,10,00,00,0,01,3,0);
                _OPN("valsound.bass3",  "Analog bass #2 (q2)",    6,4, 21,05,00,00,02,35,0,00,0,0, 26,10,00,11,01,00,0,01,0,0, 27,00,00,11,00,00,0,01,3,0, 27,14,00,11,01,00,0,01,7,0);
                _OPN("valsound.bass4",  "Chopper Bass 0",         0,5, 28,14,15,15,04,25,2,10,1,0, 31,14,10,09,03,34,2,00,2,0, 31,14,09,09,02,23,0,00,3,0, 31,06,05,11,02,00,0,00,7,0);
                _OPN("valsound.bass5",  "Chopper Bass 1",         0,5, 28,14,15,15,04,30,1,14,1,0, 31,14,10,09,02,35,1,03,2,0, 31,14,09,09,02,25,0,00,3,0, 31,06,05,11,01,00,0,00,7,0);
                _OPN("valsound.bass6",  "Chopper bass 2 (CUT)",   0,4, 31,15,28,05,02,28,3,15,6,0, 31,10,15,04,04,41,3,04,6,0, 31,08,03,05,01,21,2,00,6,0, 31,02,02,05,15,00,2,00,6,0);
                _OPN("valsound.bass7",  "Chopper bass 3",         0,5, 31,18,02,13,09,28,2,13,1,0, 31,10,15,04,04,41,3,01,2,0, 31,08,03,05,01,21,2,00,3,0, 31,02,02,12,15,00,2,00,7,0);
                _OPN("valsound.bass8",  "Elec.Chopper bass+ 4",   0,5, 31,18,02,13,09,28,2,13,1,0, 31,10,15,04,04,41,3,01,2,0, 31,08,03,05,01,21,2,00,3,0, 31,02,02,12,15,00,2,01,7,0);
                _OPN("valsound.bass9",  "Effect Bass 1",          4,3, 23,05,04,07,02,00,0,01,3,0, 30,02,02,08,02,00,1,07,3,0, 24,05,04,07,02,00,0,01,7,0, 31,02,02,08,02,00,1,10,7,0);
                _OPN("valsound.bass10", "Effect Bass 2 to UP",    4,3, 03,06,05,15,02,00,0,01,3,0, 07,04,03,15,02,00,1,07,3,0, 03,06,05,15,02,00,0,01,7,0, 07,04,03,15,02,00,1,10,7,0);
                _OPN("valsound.bass11", "Effect Bass 1",          4,3, 22,05,06,00,00,09,0,01,3,0, 19,03,04,07,01,00,0,07,3,0, 23,00,00,00,00,19,0,01,7,0, 20,02,00,07,01,00,0,01,7,0);
                _OPN("valsound.bass12", "Mohaaa",                 0,5, 07,00,00,15,00,21,0,01,0,0, 06,00,00,15,00,18,0,02,0,0, 08,00,00,15,00,23,0,01,0,0, 18,00,00,15,00,00,0,02,0,0);
                _OPN("valsound.bass13", "Effect FB Bass #5",      0,7, 31,06,02,15,03,20,0,01,3,0, 31,06,02,15,06,14,2,02,0,0, 06,06,02,15,01,08,0,01,7,0, 31,05,01,15,02,00,0,02,0,0);
                _OPN("valsound.bass14", "Magical bass",           0,7, 31,08,00,06,10,38,1,01,3,0, 28,18,05,06,13,47,1,10,7,0, 31,07,07,06,08,23,2,00,2,0, 28,09,06,08,01,00,2,00,0,0);
                _OPN("valsound.bass15", "E.Bass #6",              0,7, 31,15,00,10,05,35,1,14,3,0, 31,14,07,07,04,41,1,04,7,0, 31,14,03,00,02,18,1,00,3,0, 31,12,08,08,01,00,0,00,7,0);
                _OPN("valsound.bass16", "E.Bass #7",              3,7, 31,15,00,10,05,29,1,10,7,0, 31,13,07,07,04,46,1,04,7,0, 31,14,05,00,02,19,1,00,3,0, 31,12,04,08,01,00,0,00,0,0);
                _OPN("valsound.bass17", "E.Bass 70",              2,5, 31,08,00,00,03,34,0,00,3,0, 31,14,06,09,02,42,0,08,0,0, 31,16,03,00,02,20,0,00,7,0, 31,12,05,08,02,00,0,01,0,0);
                _OPN("valsound.bass18", "VAL006 Bass like Euro",  0,4, 31,07,07,11,02,25,3,06,0,0, 31,06,06,11,01,55,3,04,7,0, 31,09,06,11,01,18,2,00,3,0, 31,06,08,11,15,00,2,01,0,0);
                _OPN("valsound.bass19", "E.Bass x2",              2,7, 31,14,08,03,01,33,0,00,1,0, 31,17,08,09,05,30,0,14,2,0, 31,15,08,05,05,35,0,04,3,0, 31,15,08,06,01,00,0,01,7,0);
                _OPN("valsound.bass20", "E.Bass x4",              2,7, 31,14,03,00,02,33,1,03,3,0, 23,16,04,12,03,30,2,10,0,0, 31,13,03,11,03,27,1,00,7,0, 31,07,05,09,01,00,3,01,0,0);
                _OPN("valsound.bass21", "Metal pick bass X5",     3,7, 31,14,00,06,13,51,2,13,0,0, 31,13,00,06,13,21,0,01,0,0, 31,09,00,06,13,23,0,00,0,0, 31,09,00,07,13,00,0,00,0,0);
                _OPN("valsound.bass22", "Groove Bass 1",          5,3, 31,00,00,00,00,38,0,00,0,0, 21,00,00,13,00,05,0,00,0,0, 21,00,00,13,00,03,0,01,0,0, 21,00,00,13,00,03,0,01,0,0);
                _OPN("valsound.bass23", "Analog Bass Groove #2",  6,5, 31,00,00,00,00,41,0,02,0,0, 31,00,00,10,00,02,0,01,0,0, 31,00,00,10,00,01,0,01,7,0, 31,00,00,10,00,01,0,01,3,0);
                _OPN("valsound.bass24", "Harmonics #1",           6,6, 31,12,09,00,02,45,1,02,3,0, 31,10,08,06,01,00,1,01,7,0, 31,13,08,05,02,00,1,02,7,0, 31,16,12,05,10,00,1,00,3,0);
                _OPN("valsound.bass25", "Low Bass x1",            5,3, 31,00,09,15,00,25,2,00,0,0, 31,15,06,08,01,00,1,00,1,0, 31,15,06,08,01,00,1,01,2,0, 31,15,00,07,02,00,1,01,3,0);
                _OPN("valsound.bass26", "Low_bass x2 Little FB",  5,6, 21,00,09,00,00,24,2,00,0,0, 21,15,06,08,01,00,1,00,1,0, 21,15,06,08,01,00,1,01,2,0, 27,15,00,07,02,00,1,01,3,0);
                _OPN("valsound.bass27", "Low Bass x1 Rezzo.",     5,3, 31,00,09,15,00,30,2,00,0,0, 31,15,06,12,01,02,1,00,1,0, 31,15,06,12,01,02,1,01,2,0, 31,15,10,12,02,02,1,04,3,0);
                _OPN("valsound.bass28", "Low Bass Picked",        5,7, 31,05,00,00,11,33,1,00,0,0, 30,12,04,09,01,00,1,00,0,0, 27,14,08,09,03,00,1,01,0,0, 27,14,07,12,15,06,1,05,0,0);
                _OPN("valsound.bass29", "metal bass",             0,5, 20,10,09,15,01,22,0,00,7,0, 17,09,00,00,02,22,0,01,7,0, 21,09,00,00,01,18,0,00,3,0, 18,08,00,08,01,00,0,01,3,0);
                _OPN("valsound.bass30", "e.n.bass 1",             3,7, 27,14,00,04,04,25,0,07,0,0, 31,12,00,04,03,45,0,02,0,0, 31,19,00,04,05,15,0,00,0,0, 31,12,06,07,01,00,0,00,0,0);
                _OPN("valsound.bass31", "psg bass 1",             5,7, 31,14,00,00,00,22,0,00,0,0, 31,14,03,08,05,00,0,01,3,0, 31,14,03,08,03,00,0,00,0,0, 31,16,03,08,03,00,0,01,7,0);
                _OPN("valsound.bass32", "psg bass 2",             5,7, 31,14,00,00,00,22,0,01,0,0, 31,14,03,08,05,00,0,02,3,0, 31,14,03,08,03,00,0,00,0,0, 31,16,03,08,03,00,0,01,7,0);
                _OPN("valsound.bass33", "rezonance type bass #1", 2,0, 24,19,02,13,10,33,1,03,3,0, 26,16,05,14,06,28,1,00,0,0, 15,14,06,08,05,14,1,00,0,0, 31,07,05,09,02,00,0,02,7,0);
                _OPN("valsound.bass34", "slap bass",              2,2, 31,10,07,08,02,33,0,00,7,0, 21,08,08,07,05,23,3,07,7,0, 31,05,06,07,01,37,0,00,3,0, 31,08,06,07,05,00,0,01,7,0);
                _OPN("valsound.bass35", "slap bass 1",            2,7, 31,14,07,08,02,33,0,00,7,0, 21,15,06,07,04,18,2,06,7,0, 31,05,06,07,01,40,0,00,3,0, 31,12,07,07,05,00,0,01,3,0);
                _OPN("valsound.bass36", "slap bass 2 (1+)",       2,7, 31,14,07,08,02,33,0,00,7,0, 21,15,06,07,04,28,2,07,7,0, 31,05,06,07,01,40,0,00,3,0, 31,12,07,07,05,00,0,01,3,0);
                _OPN("valsound.bass37", "slap bass #3",           2,7, 31,14,07,00,05,32,0,03,7,0, 31,16,01,12,04,35,0,10,0,0, 31,11,02,00,03,23,1,00,3,0, 31,12,05,07,01,00,0,00,0,0);
                _OPN("valsound.bass38", "slap bass pull",         2,2, 31,10,07,08,02,33,0,00,7,0, 21,08,08,09,05,23,3,10,7,0, 31,05,06,10,01,37,0,00,3,0, 31,16,06,11,01,00,0,01,7,0);
                _OPN("valsound.bass39", "slap bass mute",         2,2, 31,18,07,11,12,33,0,00,7,0, 21,11,08,11,15,23,3,07,7,0, 31,15,06,11,11,37,0,00,3,0, 31,15,06,13,11,00,0,01,7,0);
                _OPN("valsound.bass40", "slap bass pick",         2,2, 31,10,07,08,02,33,0,00,7,0, 21,09,08,07,05,23,3,07,7,0, 31,05,06,08,01,37,0,00,3,0, 31,11,06,10,05,00,0,01,7,0);
                _OPN("valsound.bass41", "super bass #2",          2,2, 24,18,02,13,09,12,2,03,3,0, 26,16,05,14,09,24,1,01,0,0, 31,12,02,08,03,22,1,00,7,0, 31,07,05,09,02,00,0,01,0,0);
                _OPN("valsound.bass42", "sp_bass#3 soft",         2,3, 24,18,02,13,09,25,2,03,3,0, 26,16,05,14,09,24,1,01,0,0, 31,12,02,08,03,32,1,00,7,0, 31,07,05,09,02,00,0,01,0,0);
                _OPN("valsound.bass43", "sp_bass#4 soft*2",       2,1, 24,18,02,13,10,28,2,03,3,0, 26,16,05,14,06,24,1,00,0,0, 31,12,02,08,03,30,1,00,7,0, 31,07,05,09,02,00,0,01,0,0);
                _OPN("valsound.bass44", "sp_bass#5 attack",       0,5, 19,18,02,15,10,30,2,00,3,0, 31,16,05,14,05,24,1,00,0,0, 31,12,02,08,03,30,1,00,7,0, 31,10,07,09,02,00,0,01,0,0);
                _OPN("valsound.bass45", "sp.bass#6 rezz+pipebass",2,3, 24,18,02,13,09,35,2,12,3,0, 26,16,05,14,09,25,1,02,0,0, 31,12,02,08,03,32,1,00,7,0, 31,07,05,09,02,00,0,01,0,0);
                _OPN("valsound.bass46", "synth bass 1",           4,3, 30,00,00,00,00,23,0,01,3,0, 27,04,00,07,01,00,0,01,3,0, 30,00,00,00,00,18,0,01,7,0, 25,04,00,07,01,00,0,01,7,0);
                _OPN("valsound.bass47", "synth bass 2 myon",      5,7, 14,00,09,12,00,26,2,00,0,0, 14,15,06,08,01,00,1,00,1,0, 20,15,06,08,01,00,1,01,2,0, 18,15,00,12,02,00,1,01,3,0);
                _OPN("valsound.bass48", "synth bass #3 cho!",     3,7, 31,11,09,00,04,32,1,01,3,0, 31,15,07,08,05,41,1,08,7,0, 26,18,07,10,06,04,1,00,3,0, 31,09,06,07,01,00,0,00,7,0);
                _OPN("valsound.bass49", "synth-wind-bass #4",     2,7, 31,13,09,00,04,32,1,00,3,0, 31,15,07,08,04,21,1,01,0,0, 26,18,07,08,03,21,1,01,7,0, 31,09,06,07,01,00,0,00,0,0);
                _OPN("valsound.bass50", "synth bass #5 q2",       4,4, 20,00,00,08,07,17,0,00,3,0, 18,07,04,11,00,00,0,01,3,0, 18,00,00,09,00,22,0,01,7,0, 15,00,00,11,01,00,0,01,7,0);
                _OPN("valsound.bass51", "old wood bass",          5,7, 31,15,00,13,02,28,1,00,0,0, 31,10,01,12,01,04,0,02,0,0, 25,10,01,12,01,04,0,01,0,0, 31,10,01,12,01,04,0,02,0,0);
                _OPN("valsound.bass52", "w.bass bright",          2,7, 31,13,03,10,02,32,1,00,7,0, 31,12,04,10,03,20,1,00,0,0, 31,17,00,10,07,15,1,02,3,0, 31,06,01,10,05,00,1,01,0,0);
                _OPN("valsound.bass53", "w.bass x2 bow",          4,5, 31,11,05,00,03,25,1,00,3,0, 31,10,09,08,04,00,1,01,3,0, 23,12,05,00,04,14,1,00,7,0, 31,12,09,07,05,00,1,02,7,0);
                _OPN("valsound.bass54", "wood bass 3 (muted1)",   5,5, 31,15,00,15,02,38,1,00,0,0, 31,10,01,12,01,04,0,02,0,0, 25,10,01,12,01,02,0,01,0,0, 31,10,01,12,01,04,0,01,0,0);


                _categoly("valsound.bell");
                _OPN("valsound.bell1",  "Calm Bell",              4,3, 31,12,00,10,05,38,0,06,3,0, 31,08,04,06,11,04,0,02,3,0, 31,12,04,06,02,40,1,06,7,0, 31,06,04,06,11,00,0,02,7,0);
                _OPN("valsound.bell2",  "China Bell Double",      4,7, 21,15,08,00,03,27,1,08,3,0, 31,13,05,06,04,00,1,04,3,0, 21,15,08,00,03,25,1,06,7,0, 31,13,05,06,04,00,1,03,7,0);
                _OPN("valsound.bell3",  "Church Bell 2",          4,0, 26,03,00,02,15,35,2,04,3,0, 31,06,00,03,15,07,1,11,0,0, 31,06,00,01,14,41,2,06,7,0, 31,07,00,03,15,00,0,11,7,0);
                _OPN("valsound.bell4",  "Church Bell",            4,0, 26,03,00,02,15,35,2,04,3,0, 31,06,00,03,15,07,1,11,0,0, 31,06,00,01,14,41,2,04,7,0, 31,07,00,03,15,00,0,15,7,0);
                _OPN("valsound.bell5",  "Glocken 1",              4,3, 31,24,00,12,15,32,0,14,2,0, 31,15,00,08,15,00,0,02,0,0, 31,20,00,04,15,27,0,15,0,0, 31,14,00,05,15,00,0,02,0,0);
                _OPN("valsound.bell6",  "Harp #1",                1,7, 31,10,10,06,05,26,0,03,0,0, 31,10,10,07,05,50,1,02,0,0, 31,13,10,07,13,40,0,02,0,0, 31,14,05,07,10,00,0,01,0,0);
                _OPN("valsound.bell7",  "Harp #2",                1,3, 31,09,00,00,15,40,2,06,3,0, 31,11,00,08,15,30,2,01,7,0, 31,08,00,00,15,40,2,01,0,0, 31,08,00,08,14,00,2,01,0,0);
                _OPN("valsound.bell8",  "kirakira",               1,7, 21,11,06,00,12,31,2,06,2,0, 21,12,08,00,12,26,2,10,6,0, 28,11,07,00,12,32,1,02,0,0, 28,04,02,04,05,00,1,02,0,0);
                _OPN("valsound.bell9",  "Marimba",                4,6, 22,16,07,03,15,36,2,15,3,0, 16,10,13,07,10,00,2,01,3,0, 19,18,07,03,08,26,1,06,7,0, 16,11,12,07,10,03,2,02,7,0);
                _OPN("valsound.bell10", "Old Bell",               4,6, 27,04,00,05,14,34,0,03,3,0, 31,07,00,06,14,00,1,01,0,0, 31,07,00,03,13,41,0,14,7,0, 31,08,00,06,14,16,0,04,7,0);
                _OPN("valsound.bell11", "Percus. Bell",           5,3, 31,12,00,09,05,38,0,12,0,0, 31,15,04,05,11,09,0,03,0,0, 31,12,04,08,12,09,0,02,3,0, 31,06,04,08,11,09,0,01,7,0);
                _OPN("valsound.bell12", "Pretty Bell",            6,6, 31,12,09,00,03,43,1,02,0,0, 31,09,08,06,03,00,1,01,3,0, 31,13,08,05,03,00,1,04,7,0, 31,16,16,05,13,13,1,15,0,0);
                _OPN("valsound.bell13", "Synth Bell #0 (fromOPM)",6,2, 31,05,05,05,02,30,0,07,7,0, 31,08,05,07,15,00,0,03,7,0, 31,06,07,07,05,00,0,00,3,0, 31,08,05,05,02,10,0,01,3,0);
                _OPN("valsound.bell14", "Synth Bell #1 o5",       6,3, 31,05,05,05,02,33,1,08,3,0, 27,11,00,06,15,00,1,02,3,0, 31,06,07,06,05,00,2,00,7,0, 31,11,08,06,03,00,1,01,7,0);
                _OPN("valsound.bell15", "Synth Bell 2",           6,5, 31,08,09,00,05,33,1,07,3,0, 31,09,07,08,02,00,1,03,7,0, 31,12,07,08,01,00,1,02,3,0, 31,09,07,07,01,00,0,01,7,0);
                _OPN("valsound.bell16", "Viberaphon AMS-modu.",   4,5, 24,14,00,07,15,50,1,12,3,0, 24,10,00,07,15,00,1,04,7,0, 26,14,00,06,15,57,1,04,7,0, 26,08,00,06,15,00,2,04,3,0);
                _OPN("valsound.bell17", "twin marinba 2 g&amp, c",4,5, 18,09,05,14,12,33,1,14,3,0, 31,16,06,09,07,00,1,04,3,0, 18,09,05,14,12,33,1,07,7,0, 31,16,06,09,07,00,1,03,7,0);
                _OPN("valsound.bell18", "twin marinba 1 g&amp, c",4,2, 31,10,05,00,12,30,1,08,3,0, 31,16,06,09,09,00,1,04,3,0, 31,10,05,00,12,30,1,06,7,0, 31,16,06,09,09,00,1,03,7,0);

                _categoly("valsound.brass");
                _OPN("valsound.brass1",  "Brass strings",         5,7, 20,00,00,00,00,27,0,01,0,0, 15,03,00,06,01,05,0,02,1,0, 14,04,00,06,01,05,0,01,2,0, 15,04,00,06,01,05,0,01,3,0);
                _OPN("valsound.brass2",  "E.mute Trampet",        2,7, 13,06,00,08,01,26,2,02,3,0, 15,08,00,08,01,32,1,02,7,0, 21,15,00,08,11,20,0,02,3,0, 18,04,00,08,02,00,1,08,0,0);
                _OPN("valsound.brass3",  "HORN 2",                4,7, 15,11,02,00,02,23,0,02,3,0, 13,12,02,15,02,00,0,02,3,0, 15,13,05,00,01,27,0,02,7,0, 13,11,02,15,02,00,0,02,7,0);
                _OPN("valsound.brass4",  "Alpine Horn #3",        5,7, 15,10,00,06,05,35,0,01,0,0, 15,05,00,08,02,06,0,02,2,0, 15,05,00,08,02,06,0,01,5,0, 15,05,00,08,02,06,0,01,0,0);
                _OPN("valsound.brass5",  "Lead brass",            2,7, 18,04,02,08,01,27,0,02,3,0, 14,14,00,08,05,33,1,08,0,0, 20,00,02,08,00,36,0,02,7,0, 17,04,01,08,03,00,0,02,0,0);
                _OPN("valsound.brass6",  "Normal HORN",           4,7, 15,13,02,15,01,36,0,02,3,0, 13,12,02,15,03,00,0,02,3,0, 15,13,05,15,02,25,0,02,7,0, 13,11,02,15,03,00,0,02,7,0);
                _OPN("valsound.brass7",  "Synth Oboe",            6,3, 17,15,15,03,15,15,0,01,7,0, 16,00,09,00,00,00,0,06,3,0, 21,15,11,01,04,04,0,04,3,0, 18,15,11,01,04,04,0,06,7,0);
                _OPN("valsound.brass8",  "Oboe 2",                2,5, 19,18,00,09,02,23,1,01,0,0, 31,17,00,06,03,28,0,06,0,0, 31,20,00,05,01,51,0,08,0,0, 16,31,00,11,00,00,1,04,0,0);
                _OPN("valsound.brass9",  "Attack Brass (q2)",     4,4, 15,09,08,08,02,14,1,04,7,0, 18,15,01,08,03,00,0,04,3,0, 16,09,08,08,02,12,1,02,3,0, 31,15,01,08,03,00,0,02,7,0);
                _OPN("valsound.brass10", "SAX",                   2,6, 13,06,00,08,01,14,2,02,3,0, 15,08,00,08,01,30,1,10,7,0, 21,07,00,08,02,35,0,01,3,0, 18,04,00,09,02,00,1,02,0,0);
                _OPN("valsound.brass11", "Soft brass(lead)",      4,7, 16,03,00,02,01,30,0,01,5,0, 18,00,00,07,00,03,1,04,0,0, 16,00,00,02,02,35,0,01,1,0, 18,05,00,07,01,00,0,01,0,0);
                _OPN("valsound.brass12", "Synth Brass 1 OLD",     5,7, 31,07,05,10,02,28,0,01,0,0, 31,02,05,10,02,00,0,00,0,0, 31,02,05,10,02,02,0,01,0,0, 31,10,05,10,10,00,0,02,0,0);
                _OPN("valsound.brass13", "Synth Brass 2 OLD",     5,7, 31,10,01,10,02,28,1,02,0,0, 31,12,01,10,02,02,0,01,0,0, 31,12,01,10,02,04,0,02,0,0, 31,12,01,10,10,03,0,04,0,0);
                _OPN("valsound.brass14", "Synth Brass 3",         4,7, 15,09,00,09,02,22,0,02,7,0, 23,04,02,09,05,00,0,02,7,0, 14,10,00,09,02,20,0,02,3,0, 20,04,00,09,02,00,0,02,3,0);
                _OPN("valsound.brass15", "Synth Brass #4",        5,7, 20,00,00,00,00,22,0,04,0,0, 18,12,00,08,01,00,0,08,0,0, 20,12,00,08,01,00,0,04,6,0, 22,12,00,08,01,00,0,04,2,0);
                _OPN("valsound.brass16", "Syn.Brass 5(long)",     4,7, 29,02,02,00,03,28,0,02,7,0, 29,00,02,08,05,04,0,04,7,0, 21,02,02,00,02,32,0,01,3,0, 29,00,02,08,05,04,0,02,3,0);
                _OPN("valsound.brass17", "Synth Brass 6",         2,7, 30,08,08,05,03,25,0,01,1,0, 25,10,08,06,04,30,0,02,1,0, 20,10,05,06,03,40,0,01,5,0, 20,05,05,07,05,00,0,01,3,0);
                _OPN("valsound.brass18", "Trumpet",               2,7, 13,06,00,08,01,25,2,02,3,0, 15,08,00,08,01,32,1,06,7,0, 21,07,00,08,02,42,0,02,3,0, 18,04,00,08,02,00,1,02,0,0);
                _OPN("valsound.brass19", "Trumpet 2",             2,6, 13,06,00,08,01,14,2,02,3,0, 15,08,00,08,01,30,1,12,7,0, 21,07,00,08,02,38,0,02,3,0, 18,04,00,08,02,00,2,02,0,0);
                _OPN("valsound.brass20", "Twin Horn (or OL=25)",  4,6, 14,06,00,11,03,32,0,04,3,0, 16,08,00,09,02,00,0,04,3,0, 14,06,00,11,03,33,0,03,7,0, 16,08,00,09,02,00,0,03,7,0);


                _categoly("valsound.guitar");
                _OPN("valsound.guitar1",  "Guitar VeloLow",       1,3, 31,11,06,00,02,45,1,07,0,0, 31,07,05,00,05,35,1,02,0,0, 31,07,06,00,05,40,1,01,0,0, 31,13,05,05,01,00,1,01,0,0);
                _OPN("valsound.guitar2",  "Guitar VeloHigh",      1,4, 31,11,06,00,02,43,1,09,0,0, 31,07,05,00,05,35,1,02,0,0, 31,07,06,00,05,35,1,01,0,0, 31,13,06,05,01,00,1,01,0,0);
                _OPN("valsound.guitar3",  "A.Guitar #3",          1,7, 31,10,08,04,02,34,2,13,0,0, 31,09,07,04,02,36,0,02,0,0, 31,09,08,04,02,38,0,01,0,0, 31,04,02,08,02,00,1,01,0,0);
                _OPN("valsound.guitar4",  "Cutting E.Guitar",     3,5, 21,07,01,00,01,18,0,04,0,0, 24,00,04,09,01,15,0,06,0,0, 22,20,02,07,13,05,0,02,3,0, 31,12,00,06,01,00,0,02,7,0);
                _OPN("valsound.guitar5",  "Dis. Synth (old)",     5,7, 31,00,00,12,01,18,1,01,0,0, 31,00,01,12,01,04,1,02,0,0, 31,00,01,12,01,04,1,00,0,0, 31,00,01,12,01,04,1,02,0,0);
                _OPN("valsound.guitar6",  "Dra-spi-Dis.G.",       0,7, 16,15,01,03,03,26,0,03,3,0, 19,31,01,03,00,27,0,01,0,0, 26,31,01,03,00,26,0,01,5,0, 27,31,01,08,00,00,0,01,0,0);
                _OPN("valsound.guitar7",  "Dis.Guitar 3-",        1,7, 31,15,01,03,03,30,1,03,3,0, 31,00,01,10,01,25,1,01,0,0, 31,00,01,10,01,22,1,01,5,0, 31,13,01,07,01,00,1,01,0,0);
                _OPN("valsound.guitar8",  "Dis.Guitar 3+",        0,5, 31,04,00,00,01,08,0,03,0,0, 18,01,00,08,00,25,0,15,0,0, 31,04,00,00,01,23,0,07,7,0, 31,12,00,09,00,00,0,01,1,0);
                _OPN("valsound.guitar9",  "Feed-back Guitar 1",   3,7, 31,13,00,02,02,26,0,06,3,0, 18,07,04,10,05,24,0,03,3,0, 31,00,00,08,00,22,0,04,0,0, 31,00,00,07,01,00,0,02,7,0);
                _OPN("valsound.guitar10", "Hard Dis. Guitar 1",   0,5, 31,04,04,06,01,08,0,03,0,0, 18,01,04,00,01,27,0,12,0,0, 31,04,04,00,01,22,0,02,3,0, 31,12,00,08,01,00,0,02,7,0);
                _OPN("valsound.guitar11", "Hard Dis.Guitar 3",    0,5, 31,04,01,00,00,11,0,03,0,0, 18,01,04,07,00,23,0,15,0,0, 31,04,02,00,00,24,0,05,1,0, 31,12,00,07,01,00,0,01,7,0);
                _OPN("valsound.guitar12", "Dis.Guitar '94 Hard",  0,7, 31,00,00,11,00,21,0,09,7,0, 31,15,00,10,01,26,0,02,3,0, 31,05,00,08,01,25,0,01,3,0, 31,00,00,07,00,04,0,02,7,0);
                _OPN("valsound.guitar13", "New Dis.Guitar 1",     0,5, 31,05,00,00,00,20,0,02,3,0, 18,05,04,07,01,20,1,05,3,0, 31,06,05,00,00,22,0,01,7,0, 31,12,00,08,01,00,0,01,7,0);
                _OPN("valsound.guitar14", "New Dis.Guitar 2",     0,5, 31,05,00,00,00,20,0,03,3,0, 18,05,04,07,01,20,1,07,3,0, 31,06,05,00,00,22,0,01,7,0, 31,12,00,08,01,00,0,01,7,0);
                _OPN("valsound.guitar15", "New Dis.Guitar 3",     3,5, 31,05,00,00,10,08,0,03,0,0, 31,01,00,08,00,20,0,15,0,0, 31,04,00,15,00,22,0,01,3,0, 31,12,00,08,01,00,0,01,7,0);
                _OPN("valsound.guitar16", "Overdrive.G. (AL=013)",1,7, 31,05,00,00,01,30,0,03,0,0, 18,05,00,08,01,21,0,02,2,0, 31,05,04,00,01,29,0,01,6,0, 31,11,00,08,01,00,0,01,2,0);
                _OPN("valsound.guitar17", "METAL",                3,7, 26,16,07,04,08,24,2,08,7,0, 22,15,06,04,09,22,2,12,2,0, 26,09,02,07,08,43,1,03,0,0, 30,08,02,08,08,00,2,04,0,0);
                _OPN("valsound.guitar18", "Soft Dis.Guitar",      0,7, 16,15,01,09,03,26,0,06,3,0, 19,15,01,00,00,27,0,03,0,0, 26,15,02,00,02,26,0,01,5,0, 21,31,00,07,00,00,0,01,0,0);


                _categoly("valsound.lead");
                _OPN("valsound.lead1",  "Aco code",               4,4, 15,00,00,12,00,28,0,08,3,0, 17,06,01,12,01,00,1,08,3,0, 15,00,00,12,00,21,0,04,7,0, 17,06,01,12,01,00,1,04,7,0);
                _OPN("valsound.lead2",  "Analog synthe 1",        1,6, 31,10,00,08,05,18,0,10,0,0, 31,05,01,08,02,30,0,02,0,0, 31,05,01,08,02,50,0,08,0,0, 31,05,01,08,02,00,0,02,0,0);
                _OPN("valsound.lead3",  "Bosco-lead",             6,5, 28,02,02,06,00,20,0,05,7,0, 10,04,04,06,00,10,0,02,3,0, 15,02,02,06,00,00,0,03,7,0, 15,04,04,06,00,00,0,01,3,0);
                _OPN("valsound.lead4",  "Cosmo Lead",             3,7, 31,00,00,00,00,25,0,00,3,0, 15,00,00,01,00,25,0,01,7,0, 22,00,00,01,00,23,0,01,7,0, 18,00,00,06,00,00,0,01,0,0);
                _OPN("valsound.lead5",  "Cosmo Lead 2",           3,7, 31,00,00,00,00,33,0,00,3,0, 15,00,00,01,00,30,0,01,7,0, 22,00,00,01,00,28,0,00,7,0, 18,00,00,06,00,00,0,01,0,0);
                _OPN("valsound.lead6",  "Digital lead #1",        2,7, 31,00,00,00,00,26,0,01,0,0, 31,00,00,00,00,37,0,02,3,0, 31,00,00,00,00,27,0,02,7,0, 31,12,00,15,01,00,0,01,0,0);
                _OPN("valsound.lead7",  "Double sin wave",        7,4, 18,04,00,10,01,00,0,01,3,0, 18,04,00,07,01,00,0,04,3,0, 17,04,00,10,01,00,0,01,7,0, 14,04,00,07,01,00,0,04,7,0);
                _OPN("valsound.lead8",  "E.Organ 2 bright",       6,7, 31,00,00,09,00,33,0,05,7,0, 31,13,00,09,01,00,0,03,3,0, 31,00,00,09,00,03,0,02,3,0, 31,00,00,09,00,00,0,01,7,0);
                _OPN("valsound.lead9",  "E.Organ 2 (voice)",      6,3, 31,15,00,15,03,35,0,07,7,0, 31,00,00,09,00,00,0,03,3,0, 31,15,03,11,01,04,0,01,3,0, 31,15,00,11,01,04,0,02,7,0);
                _OPN("valsound.lead10", "E.Organ 4 Click",        6,3, 31,00,00,04,01,33,0,02,0,0, 31,00,00,10,01,00,0,01,3,0, 31,12,00,10,01,00,0,04,7,0, 31,16,00,12,06,00,1,08,3,0);
                _OPN("valsound.lead11", "E.Organ 5 Click",        6,2, 31,00,00,04,01,35,0,02,0,0, 31,00,00,10,01,00,0,02,3,0, 31,12,00,10,01,00,0,04,7,0, 28,16,00,14,08,00,1,08,3,0);
                _OPN("valsound.lead12", "E.Organ 6",              6,7, 31,15,00,00,01,33,0,07,7,0, 31,10,00,09,01,00,0,04,3,0, 31,00,00,09,00,03,0,01,3,0, 31,00,00,09,00,00,0,02,7,0);
                _OPN("valsound.lead13", "E.Organ 7 Church",       6,7, 31,00,00,09,00,33,0,04,7,0, 31,00,00,09,00,00,0,04,3,0, 31,00,00,09,00,00,0,02,3,0, 31,00,00,09,00,00,0,01,7,0);
                _OPN("valsound.lead14", "Metal Lead",             0,7, 25,05,00,15,04,22,0,02,3,0, 21,02,00,12,03,26,0,07,0,0, 18,07,05,08,04,27,0,06,7,0, 21,05,03,08,02,00,0,04,0,0);
                _OPN("valsound.lead15", "Metal Lead 3",           2,7, 31,10,00,00,01,25,0,04,3,0, 31,05,00,04,15,25,0,00,3,0, 31,09,00,06,10,37,0,04,7,0, 31,00,00,09,00,00,0,02,7,0);
                _OPN("valsound.lead16", "MONO Lead",              3,7, 24,11,01,00,08,42,2,04,2,0, 24,09,01,00,05,19,2,04,6,0, 23,09,02,00,10,25,2,08,1,0, 23,05,03,11,08,00,1,01,0,0);
                _OPN("valsound.lead17", "PSG like PC88 (long)",   1,7, 31,00,00,15,00,27,0,02,0,0, 31,00,00,15,00,50,0,01,0,0, 31,00,00,15,00,40,0,02,0,0, 31,00,00,15,00,00,0,01,0,0);
                _OPN("valsound.lead18", "PSG Cut 1",              5,7, 31,00,00,00,00,30,0,02,0,0, 31,15,00,15,03,00,0,01,0,0, 31,15,00,15,03,00,0,01,0,0, 31,15,00,15,03,00,0,01,0,0);
                _OPN("valsound.lead19", "Attack Synth",           0,7, 31,15,01,00,01,40,0,08,0,0, 31,15,01,00,01,20,1,04,0,0, 31,15,01,00,01,37,0,01,0,0, 31,15,01,08,03,00,0,01,0,0);
                _OPN("valsound.lead20", "Sin wave",               6,2, 31,00,00,15,00,43,0,00,0,0, 31,00,00,15,00,00,0,00,0,0, 31,00,00,15,00,00,0,00,0,0, 31,00,00,15,00,00,0,00,0,0);
                _OPN("valsound.lead21", "Synth &amp,  Bell 2",    4,7, 21,00,01,11,00,29,1,02,3,0, 14,08,00,13,01,08,0,04,3,0, 31,11,00,00,02,35,0,14,3,0, 31,08,05,10,15,00,0,04,7,0);
                _OPN("valsound.lead22", "Chorus #2(Voice)+bell",  4,7, 21,00,01,11,00,35,1,02,3,0, 14,08,00,13,01,00,0,02,3,0, 31,12,00,00,02,44,0,14,3,0, 31,09,05,10,15,00,0,08,7,0);
                _OPN("valsound.lead23", "Synth Cut 8-4",          4,7, 31,00,00,00,00,30,1,08,3,0, 18,13,09,07,01,00,1,08,3,0, 31,00,00,00,00,22,1,04,7,0, 21,13,09,07,01,00,1,04,7,0);
                _OPN("valsound.lead24", "Synth long 8-4",         4,7, 31,00,00,00,00,30,1,08,3,0, 18,13,01,07,01,00,1,08,3,0, 31,00,00,00,00,22,1,04,7,0, 21,13,01,07,01,00,1,04,7,0);
                _OPN("valsound.lead25", "ACO_Code #2",            4,7, 31,00,00,00,00,28,0,04,3,0, 31,10,00,07,01,00,0,04,3,0, 31,00,00,00,00,21,0,04,7,0, 31,10,00,07,01,00,0,04,7,0);
                _OPN("valsound.lead26", "ACO_Code #3",            4,7, 31,00,00,00,00,28,0,04,3,0, 31,10,00,07,01,00,0,08,3,0, 31,00,00,00,00,21,0,04,7,0, 31,10,00,07,01,00,0,04,7,0);
                _OPN("valsound.lead27", "Synth FB long 4",        3,7, 25,07,00,00,05,23,0,02,7,0, 17,00,00,09,00,32,0,04,3,0, 25,07,00,00,06,27,0,02,3,0, 16,08,00,09,01,00,0,02,0,0);
                _OPN("valsound.lead28", "Synth FB long 5",        4,7, 22,04,00,00,03,22,0,02,3,0, 16,08,00,09,02,00,0,08,3,0, 22,00,00,00,00,15,0,02,7,0, 16,08,00,09,02,00,0,08,7,0);
                _OPN("valsound.lead29", "Synth Lead 0",           4,6, 24,07,01,00,00,23,0,01,3,0, 23,08,00,06,01,00,0,01,3,0, 24,07,01,00,00,12,0,01,7,0, 15,08,00,08,01,08,0,03,7,0);
                _OPN("valsound.lead30", "Synth Lead 1",           3,7, 14,10,00,15,01,25,0,02,0,0, 31,00,07,15,00,15,0,01,3,0, 31,00,00,15,00,30,0,02,7,0, 31,00,00,15,00,00,0,01,0,0);
                _OPN("valsound.lead31", "Synth Lead 2",           2,7, 31,04,02,08,01,25,0,04,3,0, 14,14,00,08,05,32,1,04,0,0, 21,00,02,08,00,35,0,02,7,0, 21,04,01,08,03,00,0,02,0,0);
                _OPN("valsound.lead32", "Synth Lead 3",           3,7, 20,00,00,00,00,29,0,02,3,0, 18,12,00,08,01,25,0,02,7,0, 20,12,00,08,01,30,0,01,3,0, 22,12,00,08,01,00,0,02,0,0);
                _OPN("valsound.lead33", "Synth Lead 4",           4,5, 25,31,01,03,01,10,0,02,3,0, 31,10,01,10,02,00,0,04,7,0, 25,31,01,03,01,05,1,02,7,0, 31,10,01,10,02,00,0,04,3,0);
                _OPN("valsound.lead34", "Synth Lead 5",           4,6, 31,10,00,08,02,16,0,11,7,0, 31,03,00,08,02,18,0,01,7,0, 31,03,00,08,02,50,0,08,3,0, 31,03,00,08,02,00,0,02,3,0);
                _OPN("valsound.lead35", "Synth Lead 6",           4,5, 31,00,00,00,00,22,0,02,7,0, 18,10,00,06,01,00,0,08,7,0, 31,00,00,00,00,23,0,04,3,0, 18,10,00,06,01,00,0,04,3,0);
                _OPN("valsound.lead36", "Synth Lead 7 (Soft FB)", 1,7, 31,00,00,00,00,23,0,02,1,0, 31,10,08,00,05,20,0,02,7,0, 15,12,00,12,02,36,0,06,5,0, 18,00,00,06,00,00,0,02,0,0);
                _OPN("valsound.lead37", "Synth PSG",              0,7, 31,01,03,00,15,21,0,02,3,0, 31,01,06,00,15,41,0,04,3,0, 31,01,03,00,15,22,0,01,3,0, 31,13,00,06,02,00,0,01,3,0);
                _OPN("valsound.lead38", "Synth PSG 2",            0,7, 17,01,03,08,15,32,0,08,3,0, 19,01,06,08,15,35,0,04,3,0, 22,01,03,08,15,20,0,02,3,0, 31,11,00,08,02,00,0,01,3,0);
                _OPN("valsound.lead39", "Synth PSG 3",            5,7, 31,00,00,00,00,24,0,02,0,0, 31,15,00,09,03,06,0,01,0,0, 31,15,00,09,03,06,0,01,0,0, 31,15,00,09,03,06,0,02,0,0);
                _OPN("valsound.lead40", "Synth PSG 4",            5,7, 31,00,00,00,00,22,0,01,0,0, 31,15,01,09,03,00,0,01,0,0, 31,15,01,09,03,00,0,00,0,0, 31,15,02,09,04,10,0,02,0,0);
                _OPN("valsound.lead41", "Synth PSG 5",            1,7, 31,00,00,15,00,28,0,05,0,0, 31,00,00,15,00,45,0,03,0,0, 31,00,00,15,00,45,0,02,0,0, 31,00,00,15,00,00,0,01,0,0);
                _OPN("valsound.lead42", "Sin water synth",        6,0, 31,00,00,15,00,44,0,01,0,0, 24,00,00,15,00,02,0,01,0,0, 25,21,00,15,15,14,0,06,0,0, 24,00,00,15,00,04,0,02,0,0);


                _categoly("valsound.percus");
                _OPN("valsound.percus1",  "Bass Drum 2",          0,0, 30,26,00,13,15,26,0,01,0,0, 30,28,00,14,15,37,0,15,3,0, 30,16,00,08,15,05,0,00,0,0, 29,16,00,08,15,00,0,00,0,0);
                _OPN("valsound.percus2",  "Bass Drum 3 o1f",      2,5, 24,19,00,00,15,30,2,01,3,0, 31,18,13,14,15,30,1,00,0,0, 31,19,13,08,15,05,1,01,7,0, 31,16,15,12,15,00,1,01,0,0);
                _OPN("valsound.percus3",  "RUFINA BD o2c",        5,5, 29,20,18,15,05,11,1,00,2,0, 31,16,18,15,05,02,1,00,0,0, 31,16,17,15,03,00,1,00,0,0, 31,15,18,15,04,00,1,00,0,0);
                _OPN("valsound.percus4",  "B.D.(-vBend)",         3,7, 31,08,00,01,05,08,2,15,1,0, 31,21,04,01,12,18,2,01,0,0, 31,26,00,15,15,14,2,00,0,0, 31,13,10,15,15,00,2,00,0,0);
                _OPN("valsound.percus5",  "BD808_2(-vBend)",      6,3, 31,16,10,15,15,15,0,00,0,0, 31,15,10,15,15,00,0,00,0,0, 28,15,20,15,15,00,0,02,3,0, 26,15,20,15,15,00,0,02,7,0);
                _OPN("valsound.percus6",  "Cho cho 3 (o2e)",      4,2, 18,18,00,14,15,00,3,04,7,0, 17,17,00,14,15,00,2,02,3,0, 18,18,00,14,15,00,3,04,3,0, 17,17,00,14,15,00,2,02,7,0);
                _OPN("valsound.percus7",  "Cow-Bell 1",           3,7, 31,18,19,06,02,08,1,12,1,0, 31,18,12,06,02,35,1,07,2,0, 31,17,13,06,03,32,1,07,3,0, 31,19,15,09,01,00,0,02,7,0);
                _OPN("valsound.percus8",  "Crash Cymbal (noise)", 4,7, 31,00,00,00,00,00,0,15,7,0, 21,10,11,13,05,00,1,00,7,0, 31,00,00,14,00,00,0,08,3,0, 31,09,09,09,15,05,2,15,3,0);
                _OPN("valsound.percus9",  "Crash Noise",          0,7, 23,02,08,02,15,00,0,15,3,0, 25,02,08,02,15,14,1,12,7,0, 22,02,08,05,15,04,0,03,3,0, 23,07,08,05,15,00,0,06,7,0);
                _OPN("valsound.percus10", "Crash Noise Short",    0,7, 23,02,08,02,15,00,2,15,3,0, 25,02,08,02,15,14,3,12,7,0, 22,02,08,05,15,04,2,03,3,0, 23,07,08,05,15,00,2,06,7,0);
                _OPN("valsound.percus11", "ETHNIC Percus.0",      3,7, 31,19,06,03,13,40,1,10,3,0, 31,12,04,00,05,34,1,04,3,0, 31,16,06,10,14,36,1,02,7,0, 31,14,06,06,15,00,1,00,0,0);
                _OPN("valsound.percus12", "ETHNIC Percus.1",      4,6, 31,16,00,05,15,35,0,00,3,0, 31,05,15,15,15,00,0,00,3,0, 31,21,00,15,11,15,0,04,1,0, 31,20,21,09,02,00,0,00,7,0);
                _OPN("valsound.percus13", "Heavy BD.",            5,0, 31,15,00,08,15,10,0,00,0,0, 31,13,00,08,15,00,0,00,0,0, 31,13,00,08,15,00,0,00,0,0, 31,24,00,09,15,20,0,01,0,0);
                _OPN("valsound.percus14", "Heavy BD2",            5,3, 31,16,10,08,15,10,0,00,0,0, 31,15,10,08,15,00,0,00,0,0, 31,10,10,08,14,00,0,00,0,0, 31,20,10,08,15,10,0,01,0,0);
                _OPN("valsound.percus15", "Heavy SD1",            4,7, 31,00,00,00,00,00,0,15,0,0, 28,00,15,13,00,00,0,04,0,0, 31,18,15,07,04,00,1,00,0,0, 31,05,12,07,00,00,0,00,0,0);
                _OPN("valsound.percus16", "Hi-Hat close 5_",      4,7, 31,00,00,00,00,00,0,15,7,0, 18,17,18,13,05,00,0,00,7,0, 31,00,00,14,00,00,0,08,3,0, 18,16,13,09,15,05,1,15,3,0);
                _OPN("valsound.percus17", "Hi-Hat close 4",       4,7, 31,11,00,00,05,00,1,15,0,0, 31,17,12,09,09,00,1,00,0,0, 31,00,00,15,00,00,1,00,0,0, 25,19,20,15,15,07,0,15,0,0);
                _OPN("valsound.percus18", "Hi-Hat close 5",       4,7, 31,00,00,00,00,00,0,15,7,0, 21,18,18,13,05,00,0,00,7,0, 31,00,00,14,00,00,0,08,3,0, 31,17,13,09,15,05,1,15,3,0);
                _OPN("valsound.percus19", "Hi-Hat Close 6 -808-", 0,7, 27,00,10,00,15,39,0,15,0,0, 31,04,10,14,15,30,0,11,0,0, 31,10,10,14,15,05,1,09,7,0, 31,19,10,15,15,00,1,09,3,0);
                _OPN("valsound.percus20", "Hi-hat #7 Metal o3-6", 4,7, 31,16,10,08,10,00,0,15,3,0, 31,18,07,12,15,00,1,01,3,0, 31,10,10,06,10,02,0,15,7,0, 21,18,07,12,15,00,1,00,7,0);
                _OPN("valsound.percus21", "Hi-Hat Close #8 o4",   4,7, 25,19,00,09,03,00,1,15,0,0, 22,19,16,14,05,00,1,01,0,0, 31,10,15,15,05,00,0,15,0,0, 31,19,19,15,02,00,1,15,0,0);
                _OPN("valsound.percus22", "Hi-hat Open o4e-g+",   4,5, 31,05,06,00,05,00,2,15,7,0, 31,10,04,07,08,19,2,03,1,0, 31,20,06,03,03,00,2,01,7,0, 31,25,06,07,10,06,3,07,0,0);
                _OPN("valsound.percus23", "Open-hat2 Metal o4c-", 4,7, 31,14,00,08,03,00,1,15,3,0, 31,15,08,12,13,00,1,07,3,0, 31,13,00,06,03,01,0,10,7,0, 31,15,11,12,12,00,1,07,7,0);
                _OPN("valsound.percus24", "Open-hat3 Metal",      4,7, 31,14,00,08,03,00,1,15,3,0, 31,15,08,12,13,00,0,07,3,0, 31,13,00,06,01,01,0,10,7,0, 31,15,11,12,07,00,0,07,7,0);
                _OPN("valsound.percus25", "Hi-Hat Open #4 o4f",   4,6, 31,15,00,09,01,00,0,15,0,0, 31,20,05,14,05,03,0,04,0,0, 31,10,09,09,01,00,0,10,0,0, 31,22,05,14,05,00,1,07,0,0);
                _OPN("valsound.percus26", "Metal ride o4c or o5c",4,5, 20,05,00,00,05,11,2,15,3,0, 18,11,09,07,11,00,2,08,3,0, 31,19,00,03,03,00,1,15,7,0, 16,12,09,07,11,00,2,07,7,0);
                _OPN("valsound.percus27", "Rim Shot #1 o3c",      0,7, 31,11,00,15,15,37,1,15,1,0, 31,12,00,15,15,40,1,10,2,0, 31,17,00,15,15,13,2,00,3,0, 31,16,00,15,15,00,2,00,7,0);
                _OPN("valsound.percus28", "Snare Drum Light",     4,7, 31,00,00,07,00,00,0,15,0,0, 31,15,15,09,02,00,0,15,0,0, 31,21,00,15,11,10,0,04,1,0, 31,19,17,09,02,00,0,00,7,0);
                _OPN("valsound.percus29", "Snare Drum Lighter",   4,6, 31,00,00,14,00,00,0,10,3,0, 31,15,15,14,01,00,0,12,7,0, 31,15,00,14,15,00,3,02,3,0, 31,15,00,14,15,00,2,00,7,0);
                _OPN("valsound.percus30", "Snare Drum 808 o2-o3", 4,7, 31,00,00,00,00,05,0,15,7,0, 31,18,17,15,01,00,0,09,3,0, 31,19,00,15,15,00,0,00,7,0, 26,21,16,15,15,00,0,00,3,0);
                _OPN("valsound.percus31", "Snare4 -808type- o2",  4,7, 31,12,00,12,04,00,1,07,3,0, 27,15,18,15,01,00,1,15,3,0, 31,20,15,12,15,11,0,01,7,0, 31,19,15,15,15,00,1,01,7,0);
                _OPN("valsound.percus32", "Snare5 o1-2(Franger)", 4,7, 31,16,00,00,02,06,0,15,7,0, 31,18,15,15,00,00,0,09,3,0, 28,20,00,15,15,00,0,00,7,0, 25,16,15,15,15,00,0,00,3,0);
                _OPN("valsound.percus33", "Tom (old)",            4,7, 31,11,00,01,15,00,0,15,3,0, 31,20,14,15,05,00,0,01,3,0, 31,16,15,05,15,48,0,00,7,0, 31,11,15,15,15,00,0,00,7,0);
                _OPN("valsound.percus34", "Synth tom 2 algo 3",   3,7, 31,04,00,01,00,00,1,15,1,0, 31,21,04,01,10,15,1,01,0,0, 31,26,00,15,15,00,1,00,0,0, 31,11,00,07,15,00,1,00,0,0);
                _OPN("valsound.percus35", "Synth (Noisy) Tom #3", 3,7, 31,20,00,00,02,00,0,15,3,0, 31,18,13,06,08,28,0,00,3,0, 31,16,09,12,05,44,0,01,7,0, 31,14,04,12,01,00,0,00,7,0);
                _OPN("valsound.percus36", "Synth Tom #3",         1,7, 31,18,10,00,01,00,0,15,3,0, 31,15,10,06,05,00,0,05,3,0, 31,17,12,12,06,37,0,01,7,0, 31,14,04,12,01,00,0,00,7,0);
                _OPN("valsound.percus37", "Synth -DX7- Tom #4",   3,7, 31,04,00,01,00,00,1,11,0,0, 31,21,04,01,06,25,1,01,3,0, 31,26,00,15,15,00,1,00,0,0, 31,11,00,07,15,00,1,00,0,0);
                _OPN("valsound.percus38", "Triangle 1 o5c",       4,5, 31,18,00,11,02,09,0,14,3,0, 31,21,07,12,04,00,0,08,3,0, 31,22,00,12,15,00,0,15,7,0, 31,20,06,15,15,00,0,07,7,0);


                _categoly("valsound.piano");
                _OPN("valsound.piano1",  "Aco Piano2 (Attack)",   4,5, 31,05,00,00,00,23,1,01,3,0, 20,10,03,07,08,00,1,01,3,0, 31,03,00,00,00,25,1,01,7,0, 31,12,03,07,10,02,1,01,7,0);
                _OPN("valsound.piano2",  "Backing 1 (Clav.)",     2,7, 31,08,03,06,02,40,2,01,3,0, 31,07,04,06,02,37,1,05,2,0, 31,07,02,06,01,30,1,03,7,0, 28,30,09,07,00,00,2,01,4,0);
                _OPN("valsound.piano3",  "Clav. coad",            2,6, 31,15,08,06,02,35,0,12,3,0, 31,06,02,06,02,32,0,03,0,0, 31,06,02,06,01,32,0,01,7,0, 31,08,06,07,04,00,0,02,0,0);
                _OPN("valsound.piano4",  "Deep Piano 1",          2,5, 31,09,04,00,02,38,0,00,3,0, 22,07,03,09,03,31,1,03,0,0, 31,07,03,02,03,27,1,00,7,0, 28,07,01,07,01,00,1,00,0,0);
                _OPN("valsound.piano5",  "Deep Piano 3",          2,0, 31,20,09,00,02,08,1,00,7,0, 31,11,03,01,01,23,1,04,3,0, 31,13,05,02,02,30,0,00,3,0, 31,00,04,06,00,00,1,01,7,0);
                _OPN("valsound.piano6",  "E.piano #2",            4,6, 22,05,00,03,05,30,0,02,3,0, 16,08,08,07,02,00,1,02,3,0, 20,05,00,03,05,34,0,04,7,0, 17,08,07,07,02,00,1,02,7,0);
                _OPN("valsound.piano7",  "E.piano #3",            4,7, 22,05,00,03,05,41,0,01,3,0, 16,08,08,07,02,00,1,02,3,0, 31,18,00,03,10,44,0,08,7,0, 31,09,07,07,02,03,1,01,7,0);
                _OPN("valsound.piano8",  "E.piano #4(2+)",        4,6, 31,05,00,15,05,46,2,02,3,0, 31,09,08,15,03,00,2,02,3,0, 31,05,00,15,05,44,2,04,7,0, 31,09,07,15,03,00,2,02,7,0);
                _OPN("valsound.piano9",  "E.(Bell)Piano #5",      4,7, 31,07,00,09,05,35,0,06,3,0, 31,11,07,14,04,05,1,02,3,0, 31,10,09,09,05,35,0,12,7,0, 31,11,07,14,04,05,0,02,7,0);
                _OPN("valsound.piano10", "E.Piano #6",            4,7, 29,20,00,00,03,34,0,08,3,0, 17,08,00,07,06,02,0,04,3,0, 30,00,00,00,00,25,0,04,7,0, 18,08,00,07,06,02,0,04,7,0);
                _OPN("valsound.piano11", "E.Piano #7",            4,7, 31,15,00,10,15,40,0,15,0,0, 31,10,00,07,15,15,0,01,0,0, 31,10,00,05,15,20,0,01,0,0, 31,10,00,07,15,00,0,01,0,0);
                _OPN("valsound.piano12", "Harpci chord 1",        2,5, 31,13,00,15,10,30,1,00,3,0, 31,11,02,00,03,32,1,07,3,0, 31,02,00,00,01,30,0,00,7,0, 31,06,06,07,01,00,1,04,7,0);
                _OPN("valsound.piano13", "Harpci 2",              2,7, 31,04,00,05,01,30,2,00,3,0, 31,09,01,02,01,40,2,12,0,0, 31,04,03,06,01,30,1,03,7,0, 31,11,05,08,04,00,2,01,0,0);
                _OPN("valsound.piano14", "Piano1 (ML1,10,05,01)", 2,7, 28,04,00,05,01,37,2,01,3,0, 22,09,01,02,01,47,2,12,0,0, 29,04,03,06,01,37,1,03,7,0, 18,08,00,06,06,00,2,01,0,0);
                _OPN("valsound.piano15", "Piano3",                2,7, 31,04,02,00,01,35,2,01,3,0, 24,00,01,05,00,38,3,01,0,0, 28,00,00,05,00,42,2,04,5,0, 28,07,04,06,04,00,2,01,0,0);
                _OPN("valsound.piano16", "Piano4",                2,7, 31,04,00,05,01,37,2,01,3,0, 31,09,01,02,01,47,2,10,0,0, 31,04,03,06,01,37,1,02,7,0, 31,08,00,06,06,00,1,01,0,0);
                _OPN("valsound.piano17", "Digital Piano #5",      3,7, 28,04,00,07,01,27,1,01,4,0, 28,14,07,04,03,42,2,14,3,0, 26,04,03,08,02,38,0,03,7,0, 25,07,08,07,00,00,2,01,6,0);
                _OPN("valsound.piano18", "Piano 6 High-tone",     2,7, 28,04,00,05,01,39,2,01,3,0, 31,13,01,02,02,50,2,14,0,0, 29,04,03,06,01,41,1,03,7,0, 21,08,06,06,06,00,2,01,0,0);
                _OPN("valsound.piano19", "Panning Harpci",        2,7, 31,02,10,06,14,40,3,12,3,0, 25,02,15,06,14,32,0,09,0,0, 30,02,04,06,14,34,3,05,7,0, 20,02,08,06,14,00,3,01,0,0);
                _OPN("valsound.piano20", "Yam Harpci chord",      1,4, 31,06,05,06,07,40,0,10,0,0, 31,06,04,05,05,35,0,02,0,0, 31,06,05,05,05,24,0,01,0,0, 31,07,06,07,05,00,0,01,0,0);

                _categoly("valsound.se");
                _OPN("valsound.se1", "S.E.(Detune is needed o2c)",0,4, 31,07,03,00,01,12,0,00,1,0, 31,10,00,08,03,25,0,00,2,0, 31,06,04,08,08,00,0,04,3,0, 31,12,00,08,00,00,0,12,7,0);
                _OPN("valsound.se2", "S.E. 2 o0-1-2",             1,2, 31,06,07,15,03,08,0,00,3,0, 31,06,09,15,06,08,0,00,7,0, 06,06,06,15,01,10,0,12,3,0, 31,05,04,15,02,00,0,00,0,0);
                _OPN("valsound.se3", "S.E. 3(Feedin /noise add.)",4,6, 04,03,00,15,04,00,0,00,7,0, 08,07,07,15,01,00,0,07,7,0, 05,00,00,15,00,00,0,00,3,0, 08,06,03,15,01,00,0,04,3,0);

                _categoly("valsound.special");
                _OPN("valsound.special1", "Digital 1",            3,6, 31,12,03,05,05,26,0,14,7,0, 31,16,06,00,03,28,0,08,3,0, 31,00,12,00,00,30,0,00,0,0, 31,15,12,12,02,00,0,01,7,0);
                _OPN("valsound.special2", "Digital 2",            0,7, 31,15,00,12,03,27,0,10,7,0, 31,16,00,00,04,30,0,15,3,0, 31,15,00,00,02,30,0,02,0,0, 31,15,00,12,02,00,0,01,7,0);
                _OPN("valsound.special3", "Digital[BAS] 3 o2-o3", 0,7, 31,00,00,09,00,27,0,12,1,0, 31,10,00,09,01,25,0,00,2,0, 31,10,00,09,01,25,0,12,3,0, 31,12,00,14,02,03,0,01,7,0);
                _OPN("valsound.special4", "Digital[GTR] 3 o2-o3", 0,7, 31,00,00,09,00,27,0,12,1,0, 31,10,00,09,01,25,0,00,2,0, 31,10,00,09,01,25,0,03,3,0, 31,12,00,14,02,03,0,01,7,0);
                _OPN("valsound.special5", "Digital 4 o4a",        5,0, 31,31,00,00,00,61,0,09,0,0, 31,31,00,13,00,03,0,03,0,0, 31,31,00,13,00,15,0,09,0,0, 31,31,00,13,00,06,0,06,0,0);


                _categoly("valsound.strpad");
                _OPN("valsound.strpad1",  "Accordion1",           4,6, 17,00,00,00,00,20,0,04,3,0, 16,09,00,12,02,00,0,08,3,0, 15,00,00,07,00,36,0,04,7,0, 15,09,00,12,02,00,0,08,7,0);
                _OPN("valsound.strpad2",  "Accordion2",           4,6, 21,00,01,11,00,22,1,04,3,0, 14,08,00,13,01,00,0,04,3,0, 21,00,01,10,00,30,1,04,7,0, 14,08,01,13,01,00,0,04,7,0);
                _OPN("valsound.strpad3",  "Accordion3",           4,7, 31,05,00,00,00,25,0,04,7,0, 14,08,00,13,01,00,0,04,7,0, 31,08,00,00,10,25,0,02,3,0, 14,06,00,13,01,00,0,04,3,0);
                _OPN("valsound.strpad4",  "Chorus #2(Voice)",     4,6, 21,00,01,11,00,40,1,04,3,0, 14,08,00,13,01,00,0,04,3,0, 21,00,01,10,00,37,1,04,7,0, 14,08,01,13,01,00,0,04,7,0);
                _OPN("valsound.strpad5",  "Chorus #3",            4,4, 21,00,00,02,00,42,0,04,3,0, 18,04,00,09,01,00,0,08,3,0, 21,00,00,02,00,45,0,04,7,0, 18,04,00,09,01,00,0,04,7,0);
                _OPN("valsound.strpad6",  "Chorus #4",            6,3, 21,00,00,02,00,39,0,04,0,0, 18,04,00,09,01,00,0,04,1,0, 18,10,00,09,01,00,0,04,3,0, 18,08,00,09,02,00,0,02,7,0);
                _OPN("valsound.strpad7",  "F.Strings 1",          2,7, 25,10,00,05,01,29,1,01,1,0, 25,11,00,08,05,15,1,05,1,0, 28,13,00,06,02,45,1,01,0,0, 14,04,00,06,00,00,1,01,0,0);
                _OPN("valsound.strpad8",  "F.Strings 2",          2,0, 21,07,00,07,03,37,1,01,3,0, 20,11,00,12,03,15,1,05,7,0, 16,08,00,12,03,45,1,01,0,0, 14,05,00,12,01,00,1,01,0,0);
                _OPN("valsound.strpad9",  "F.Strings 3",          2,7, 25,10,00,05,01,35,1,01,3,0, 25,11,00,08,05,13,1,05,0,0, 28,13,00,06,02,45,1,01,7,0, 14,04,00,06,01,00,1,01,0,0);
                _OPN("valsound.strpad10", "F.Strings 4 (low)",    2,7, 25,10,00,05,01,29,1,00,3,0, 25,11,00,08,05,20,1,04,0,0, 28,13,00,06,02,38,1,01,7,0, 14,04,00,06,01,00,1,01,0,0);
                _OPN("valsound.strpad11", "Pizzicate#1(KOTO2)",   0,6, 31,07,08,01,02,30,3,03,7,0, 31,05,09,01,01,30,3,02,0,0, 31,05,08,03,02,35,3,01,0,0, 31,11,07,05,05,00,2,01,3,0);
                _OPN("valsound.strpad12", "sound truck modoki",   4,7, 31,00,00,00,00,30,1,02,3,0, 18,13,04,07,01,00,1,02,3,0, 31,00,00,00,00,22,1,03,7,0, 21,13,04,07,01,00,1,03,7,0);
                _OPN("valsound.strpad13", "Strings",              2,7, 15,09,00,05,01,27,2,02,3,0, 15,00,00,05,15,31,2,02,0,0, 15,00,00,05,00,27,1,02,0,0, 13,03,00,08,00,00,1,02,7,0);
                _OPN("valsound.strpad14", "Synth Accordion",      4,7, 18,00,00,11,00,21,1,02,3,0, 15,09,00,13,02,00,1,08,3,0, 18,00,00,11,00,20,1,02,7,0, 14,09,00,13,02,00,1,04,7,0);
                _OPN("valsound.strpad15", "Phaser synthe.",       0,7, 27,31,05,05,05,08,0,01,4,0, 26,31,04,05,04,18,0,01,5,0, 19,31,00,05,02,18,0,01,6,0, 16,15,00,06,02,00,0,03,4,0);
                _OPN("valsound.strpad16", "FB Synth.",            3,7, 31,06,00,00,04,22,0,02,0,0, 18,00,00,08,00,28,0,04,6,0, 20,05,00,08,02,28,0,02,1,0, 20,04,00,08,01,00,0,02,2,0);
                _OPN("valsound.strpad17", "Synth Strings MB",     3,7, 21,00,00,00,00,30,0,01,1,0, 14,07,07,06,03,25,0,02,6,0, 15,00,00,05,00,38,0,05,6,0, 18,00,00,06,00,00,0,02,0,0);
                _OPN("valsound.strpad18", "Synth Strings #2",     2,7, 20,01,00,08,01,32,0,02,7,0, 15,04,00,08,01,28,0,02,0,0, 22,01,00,08,01,34,0,02,3,0, 14,02,00,09,01,00,0,02,0,0);
                _OPN("valsound.strpad19", "Synth.Sweep Pad #1",   5,6, 31,07,00,09,05,18,0,04,0,0, 31,15,00,14,01,05,0,08,0,0, 31,16,00,15,02,05,0,04,0,0, 31,26,00,14,02,05,0,08,1,0);
                _OPN("valsound.strpad20", "Twin synth. #1 Calm",  4,2, 16,03,00,00,02,25,0,04,3,0, 19,12,00,06,01,00,0,04,3,0, 16,03,00,00,02,20,0,03,7,0, 19,12,00,06,01,00,0,03,7,0);
                _OPN("valsound.strpad21", "Twin synth. #2 FB",    4,6, 16,03,00,00,02,20,0,04,7,0, 19,12,00,06,01,00,0,08,3,0, 16,03,00,00,02,06,0,03,3,0, 19,12,00,06,01,08,0,06,7,0);
                _OPN("valsound.strpad22", "Twin synth. #3 FB",    4,6, 16,06,05,00,02,20,0,04,3,0, 19,12,00,06,01,00,0,08,3,0, 16,06,05,00,02,06,0,03,7,0, 19,12,00,06,01,08,0,06,7,0);
                _OPN("valsound.strpad23", "Vocoder voice1",       4,7, 31,08,05,12,15,20,0,04,7,0, 31,00,00,15,00,00,0,08,7,0, 31,08,05,12,15,24,0,03,3,0, 31,00,00,15,00,00,0,06,3,0);
                _OPN("valsound.strpad24", "Voice o3-o5",          6,0, 10,00,01,03,00,70,0,01,0,0, 12,00,00,05,00,07,2,03,3,0, 12,00,01,06,02,00,1,02,7,0, 18,00,00,06,00,17,1,00,3,0);
                _OPN("valsound.strpad25", "Voice' o3-o5",         6,0, 10,00,01,03,00,70,0,00,0,0, 12,00,00,05,00,06,2,03,3,0, 12,00,01,06,02,00,1,02,7,0, 18,00,00,06,00,10,1,01,3,0);


                _categoly("valsound.wind");
                _OPN("valsound.wind1", "Clarinet #1",             3,7, 31,00,00,07,00,35,0,04,0,0, 25,14,00,04,02,42,0,04,0,0, 31,00,00,08,00,38,0,02,0,0, 18,07,00,08,01,00,0,01,0,0);
                _OPN("valsound.wind2", "Clarinet #2 Brighter",    3,7, 31,00,00,07,00,40,0,08,0,0, 27,17,00,04,04,45,0,08,0,0, 31,00,00,08,00,37,0,04,0,0, 18,07,00,08,01,00,0,01,0,0);
                _OPN("valsound.wind3", "E.Flute",                 3,0, 20,00,00,10,00,38,0,06,0,0, 14,16,00,10,05,36,0,02,3,0, 18,18,00,10,03,40,0,04,0,0, 14,12,00,10,01,00,1,02,0,0);
                _OPN("valsound.wind4", "E.Flute 2",               3,5, 20,00,00,10,00,28,0,02,3,0, 14,16,00,10,05,40,0,02,3,0, 18,18,00,10,03,34,0,04,0,0, 14,12,00,10,01,00,1,02,7,0);
                _OPN("valsound.wind5", "Flute + Bell",            4,5, 16,00,00,08,01,47,0,08,3,0, 14,00,00,08,00,00,0,04,3,0, 31,12,00,08,02,45,0,14,7,0, 31,08,00,08,15,00,0,08,7,0);
                _OPN("valsound.wind6", "Old flute",               2,7, 20,05,00,14,01,50,0,04,0,0, 15,15,00,14,02,45,0,08,0,0, 18,15,00,14,02,50,0,08,0,0, 14,02,00,14,00,00,0,04,0,0);
                _OPN("valsound.wind7", "Whitsle 1",               2,7, 20,05,00,14,01,60,0,04,0,0, 15,15,00,14,02,55,0,12,0,0, 18,15,00,14,02,60,0,08,0,0, 14,02,00,14,00,00,0,04,0,0);
                _OPN("valsound.wind8", "Whitsle 2",               2,7, 20,05,00,14,01,55,0,02,0,0, 15,15,00,14,02,55,0,08,0,0, 18,15,00,14,02,60,0,08,0,0, 14,02,00,14,00,00,0,04,0,0);


                _categoly("valsound.world");
                _OPN("valsound.world1", "Banjo (Harpci)",         1,7, 31,07,00,10,15,38,0,12,7,0, 31,08,06,07,03,52,2,10,1,0, 31,12,06,07,03,25,0,01,0,0, 31,11,07,07,03,00,2,03,5,0);
                _OPN("valsound.world2", "KOTO",                   3,0, 31,00,00,10,00,38,0,06,0,0, 24,13,00,10,05,40,0,02,3,0, 28,15,00,10,03,40,0,04,0,0, 24,12,08,10,02,00,1,02,0,0);
                _OPN("valsound.world3", "Koto 2",                 0,7, 31,07,03,03,02,30,3,03,7,0, 31,05,03,03,01,30,3,02,0,0, 31,05,03,05,02,30,3,01,0,0, 31,10,03,07,02,00,3,01,3,0);
                _OPN("valsound.world4", "Sitar 1",                0,6, 18,05,03,01,02,30,1,03,7,0, 31,05,04,01,01,28,1,02,0,0, 31,05,03,03,02,35,1,01,0,0, 31,10,02,05,04,00,0,01,3,0);
                _OPN("valsound.world5", "Shamisen 2",             3,7, 31,16,06,07,02,33,0,01,3,0, 31,16,06,07,04,18,2,06,0,0, 31,06,06,07,01,40,0,01,7,0, 31,15,06,07,05,00,0,02,0,0);
                _OPN("valsound.world6", "Shamisen 1",             2,7, 31,16,06,07,02,33,0,01,3,0, 31,16,06,07,04,18,2,08,0,0, 31,06,06,07,01,40,0,01,7,0, 31,15,06,07,05,00,0,02,0,0);
                _OPN("valsound.world7", "Synth Shamisen",         2,7, 31,16,06,07,01,33,0,01,3,0, 31,16,06,07,04,18,2,07,0,0, 31,06,06,07,00,40,0,01,7,0, 31,15,06,07,02,00,0,02,0,0);
                
                // 258 voices from valsound
            }


            if (includeFlag & INCLUDE_MIDI) {
                _categoly("midi");
                _MA3("midi.piano1",  "GrandPno",  3,0, 08,15,07,00,06,15,39,0,1,01,0,0, 00,14,03,02,03,02,28,1,3,05,0,0, 00,13,01,01,04,03,22,0,0,01,0,0, 00,13,03,02,06,04,00,1,2,01,0,0);
                _MA3("midi.piano2",  "BritePno",  3,0, 00,15,02,02,02,05,39,1,2,01,0,0, 00,15,02,02,03,15,28,0,2,05,0,0, 00,15,02,02,02,13,25,1,2,01,0,0, 00,15,02,01,05,04,10,1,0,01,0,0);
                _MA3("midi.piano3",  "E.GrandP",  5,6, 00,13,02,02,04,06,20,1,1,04,0,0, 00,13,01,01,06,07,08,1,2,01,0,0, 00,13,03,03,04,14,11,1,1,02,0,0, 00,13,01,01,05,15,08,1,2,02,0,0);
                _MA3("midi.piano4",  "HnkyTonk",  5,6, 00,15,01,02,05,14,26,1,0,01,3,2, 00,13,03,02,10,03,02,1,2,02,7,2, 00,12,01,02,05,03,23,0,0,01,7,0, 00,13,03,03,10,03,02,1,2,02,3,2);
                _MA3("midi.piano5",  "E.Piano1",  3,1, 00,11,03,02,10,03,27,1,1,03,0,1, 00,11,02,02,09,04,27,0,3,03,0,0, 00,10,04,01,04,01,19,1,1,02,0,0, 00,10,01,01,07,08,05,1,0,01,0,1);
                _MA3("midi.piano6",  "E.Piano2",  5,5, 18,15,04,05,12,11,35,1,0,07,0,2, 00,15,02,01,08,15,04,0,2,01,0,2, 00,15,00,01,11,01,18,1,1,01,0,2, 00,15,02,01,07,15,04,1,0,01,0,2);
                _MA3("midi.piano7",  "Harpsi.",   6,4, 04,14,02,02,05,00,00,1,1,01,0,0, 03,15,02,00,05,03,20,1,3,06,0,0, 04,15,03,00,01,06,28,1,2,07,0,0, 05,14,02,02,07,15,04,1,0,01,0,0);
                _MA3("midi.piano8",  "Clavi.",    3,5, 05,15,01,01,06,15,24,0,0,01,0,2, 00,15,01,01,05,00,29,0,0,01,0,2, 04,15,03,03,07,02,27,1,1,07,0,2, 00,15,02,02,09,02,08,1,0,03,0,2);
                
                _MA3("midi.chrom1",  "Celesta",   5,2, 02,14,06,06,05,15,21,1,2,09,0,0, 00,13,04,04,04,14,06,0,0,01,0,0, 05,14,06,06,06,12,22,1,3,11,0,0, 00,14,04,04,04,14,06,0,0,01,0,0);
                _MA3("midi.chrom2",  "Glocken",   7,0, 00,15,09,03,04,04,09,1,0,07,0,2, 00,15,11,02,03,11,15,1,0,04,0,2, 00,15,03,02,04,04,18,1,1,02,0,2, 00,15,04,03,04,14,04,0,0,01,0,2);
                _MA3("midi.chrom3",  "MusicBox",  5,0, 01,05,05,02,02,00,32,1,2,02,0,0, 00,15,04,03,02,00,01,1,1,01,3,0, 01,10,05,02,02,00,28,1,0,09,0,0, 00,15,02,01,01,00,06,1,0,01,7,0);
                _MA3("midi.chrom4",  "Vibes",     5,0, 00,12,04,02,04,02,23,0,0,07,0,2, 00,13,09,02,05,06,07,1,2,04,0,2, 00,12,04,02,03,02,30,1,0,08,0,2, 00,13,02,03,04,15,07,0,0,01,0,1);
                _MA3("midi.chrom5",  "Marimba",   5,7, 00,10,07,04,04,15,40,1,1,12,0,0, 00,11,04,04,05,15,05,0,0,01,0,0, 00,11,07,06,04,15,33,1,0,06,0,0, 00,13,04,05,05,15,05,1,0,01,0,0);
                _MA3("midi.chrom6",  "Xylophon",  5,2, 00,15,09,06,06,13,24,0,2,05,0,0, 00,15,07,05,07,13,03,0,0,01,0,0, 00,15,06,06,06,10,29,1,2,05,0,0, 00,15,06,06,07,14,00,0,2,01,0,0);
                _MA3("midi.chrom7",  "TubulBel",  5,0, 16,15,04,03,03,05,16,0,1,10,0,1, 00,15,03,02,03,02,05,0,0,01,0,0, 08,15,04,03,03,05,16,0,1,07,3,1, 00,15,03,02,03,02,05,0,2,02,7,0);
                _MA3("midi.chrom8",  "Dulcimer",  6,3, 01,14,10,04,04,12,06,0,2,02,0,0, 01,11,03,03,03,05,20,0,1,03,0,0, 00,13,03,03,03,00,10,0,0,01,0,0, 00,12,04,04,04,06,06,0,0,01,0,0);

                _MA3("midi.organ1",  "DrawOrgn",  2,0, 00,15,04,00,12,00,00,0,2,00,0,1, 04,15,05,00,12,00,00,0,1,01,0,0, 09,13,05,00,12,01,07,0,2,03,1,1, 04,15,01,00,12,00,07,0,2,02,0,1);
                _MA3("midi.organ2",  "PercOrgn",  7,4, 00,14,05,00,10,01,03,0,2,00,2,2, 00,13,08,00,00,05,29,0,0,02,0,0, 00,14,05,00,10,01,01,0,2,01,3,0, 00,14,06,00,10,00,01,0,2,02,7,0);
                _MA3("midi.organ3",  "RockOrgn",  7,4, 00,15,15,00,13,00,09,0,1,01,3,3, 21,11,15,00,10,01,05,0,1,01,0,2, 00,15,15,00,14,00,09,0,0,02,6,0, 17,15,15,00,14,00,09,0,0,00,7,1);
                _MA3("midi.organ4",  "ChrchOrg",  7,0, 00,09,15,00,05,00,19,0,2,03,0,0, 00,11,07,00,02,02,29,0,0,07,0,0, 00,08,15,00,05,00,04,0,2,01,0,0, 05,08,07,00,05,00,04,0,2,00,0,0);
                _MA3("midi.organ5",  "ReedOrgn",  5,3, 16,07,08,00,05,01,24,1,2,02,0,0, 00,05,15,00,06,00,00,1,2,01,0,0, 05,06,12,00,05,03,10,1,1,01,0,0, 00,05,15,00,07,00,00,1,1,02,0,0);
                _MA3("midi.organ6",  "Acordion",  5,2, 17,08,02,00,00,01,21,0,0,03,6,0, 00,07,02,00,10,02,02,0,1,01,7,0, 17,06,15,00,00,01,18,0,0,01,2,0, 00,07,15,00,10,00,07,0,2,02,3,0);
                _MA3("midi.organ7",  "Harmnica",  4,0, 00,15,15,00,09,00,44,0,3,14,0,0, 00,15,15,00,08,00,41,0,0,10,0,0, 00,15,15,00,08,00,36,0,0,01,0,0, 00,06,15,00,08,00,03,0,2,02,0,0);
                _MA3("midi.organ8",  "TangoAcd",  5,4, 12,07,12,00,00,00,15,0,0,02,1,1, 00,07,02,00,10,00,10,0,2,02,1,0, 05,07,15,00,00,00,20,0,0,01,0,0, 16,07,15,00,10,00,10,0,0,01,0,1);

                _MA3("midi.guitar1", "NylonGtr",  5,6, 00,14,01,01,04,08,21,1,1,01,0,0, 00,15,03,03,07,15,00,0,0,01,0,0, 01,11,05,05,05,04,14,1,0,03,0,0, 00,13,04,04,09,15,13,0,2,01,0,0);
                _MA3("midi.guitar2", "SteelGtr",  4,4, 05,15,07,01,04,02,26,0,2,09,0,2, 00,15,03,01,08,05,45,1,2,13,0,2, 00,15,02,01,04,01,23,1,2,01,0,2, 00,13,03,02,08,15,04,0,0,01,0,1);
                _MA3("midi.guitar3", "Jazz Gtr",  3,0, 00,15,07,01,07,03,17,1,1,01,0,2, 00,15,05,01,04,02,18,0,3,05,0,2, 00,15,02,00,07,15,31,0,1,03,0,2, 00,12,02,00,08,15,04,0,0,01,0,2);
                _MA3("midi.guitar4", "CleanGtr",  5,0, 01,15,10,02,02,01,15,0,1,01,0,0, 00,15,02,02,09,15,03,0,2,01,0,0, 04,15,02,02,03,06,16,1,1,03,0,0, 00,14,04,04,08,06,03,0,2,01,0,0);
                _MA3("midi.guitar5", "Mute.Gtr",  5,7, 00,13,08,06,07,07,17,1,0,00,0,0, 00,14,09,03,09,07,00,0,0,00,0,0, 00,14,03,03,08,09,04,0,3,01,0,0, 00,11,04,03,10,03,00,1,0,01,0,0);
                _MA3("midi.guitar6", "Ovrdrive",  4,2, 12,15,08,00,02,15,19,1,0,00,7,2, 00,12,01,00,01,01,15,0,1,02,3,2, 00,11,02,00,10,01,15,0,0,01,0,2, 00,11,01,01,10,01,10,0,0,02,0,2);
                _MA3("midi.guitar7", "Dist.Gtr",  4,4, 03,11,12,00,02,00,08,0,1,02,0,0, 00,12,05,00,10,01,29,0,0,01,0,0, 08,12,05,00,10,01,23,0,0,02,0,0, 06,12,01,00,10,05,15,0,0,01,0,0);
                _MA3("midi.guitar8", "GtrHarmo",  5,5, 08,15,02,08,07,00,22,0,1,00,0,0, 08,13,03,03,09,15,06,0,0,02,0,0, 00,11,02,08,07,00,17,0,0,00,0,0, 06,10,07,07,07,15,13,0,0,02,0,0);

                _MA3("midi.bass1",   "Aco.Bass",  5,3, 00,11,03,03,08,10,14,1,1,01,0,0, 00,12,03,03,08,11,00,0,0,01,0,0, 00,09,03,03,01,01,07,0,3,01,0,0, 00,12,03,03,08,10,05,0,0,01,0,0);
                _MA3("midi.bass2",   "FngrBass",  3,6, 00,10,02,01,03,01,28,1,2,01,0,2, 00,09,04,03,06,04,58,1,0,12,0,2, 00,11,03,02,03,02,22,1,2,01,0,2, 00,11,01,01,08,02,00,1,0,02,0,2);
                _MA3("midi.bass3",   "PickBass",  3,5, 00,15,07,02,03,01,19,1,2,01,0,2, 00,12,11,04,06,07,21,1,0,07,0,2, 00,15,09,02,06,02,23,1,2,02,0,2, 00,11,02,06,08,06,00,1,0,01,0,2);
                _MA3("midi.bass4",   "Fretless",  3,4, 00,12,03,02,03,01,29,1,2,01,2,2, 00,10,03,03,06,03,25,1,2,01,1,2, 00,09,03,02,06,01,25,1,2,01,0,2, 00,11,01,02,08,02,00,1,0,02,0,2);
                _MA3("midi.bass5",   "SlapBas1",  3,3, 00,15,07,02,03,02,14,1,2,01,0,2, 00,15,06,06,06,04,21,1,0,09,0,2, 00,12,09,02,06,02,24,1,2,01,0,2, 00,15,02,15,08,15,03,1,0,01,0,2);
                _MA3("midi.bass6",   "SlapBas2",  3,2, 00,15,07,02,03,01,14,1,2,01,0,2, 00,11,05,06,07,02,18,0,0,13,0,2, 00,09,09,02,06,02,30,1,2,01,0,2, 00,15,02,06,08,06,06,1,0,01,0,2);
                _MA3("midi.bass7",   "SynBass1",  3,5, 00,14,06,02,08,05,14,0,0,01,0,0, 08,14,04,01,08,06,39,0,1,02,0,0, 00,14,02,01,08,06,35,0,0,01,0,0, 00,14,02,02,08,09,00,0,0,01,0,0);
                _MA3("midi.bass8",   "SynBass2",  5,6, 00,15,05,07,08,06,20,0,0,02,0,2, 00,15,01,07,08,12,00,0,1,02,0,2, 00,15,03,07,07,06,20,0,0,01,0,2, 00,15,02,07,08,12,00,0,1,01,0,2);
                
                _MA3("midi.strings1", "Violin",    5,2, 12,06,00,00,03,00,18,1,2,01,0,0, 00,06,04,00,07,02,03,0,0,01,5,0, 06,14,05,07,10,00,06,1,1,04,0,0, 00,06,07,07,07,15,03,0,2,01,1,0);
                _MA3("midi.strings2", "Viola",     5,2, 01,06,00,00,03,00,09,1,2,01,0,0, 00,06,06,00,07,01,03,0,0,01,0,0, 06,14,06,07,07,00,08,1,1,01,0,0, 00,06,07,07,07,15,03,0,2,01,0,0);
                _MA3("midi.strings3", "Cello",     3,4, 01,15,06,00,06,00,16,0,0,01,0,0, 00,15,05,15,14,15,20,0,0,05,0,2, 00,15,05,00,07,02,45,0,0,01,0,0, 00,06,03,00,07,01,01,0,2,03,0,0);
                _MA3("midi.strings4", "ContraBs",  3,6, 00,15,06,00,02,00,25,0,0,01,0,0, 17,15,06,15,14,15,21,0,0,05,0,2, 00,15,06,00,04,03,27,0,0,03,0,0, 00,06,03,00,07,01,00,0,2,02,0,0);
                _MA3("midi.strings5", "Trem.Str",  5,3, 20,07,02,00,03,01,22,1,2,01,2,0, 00,06,03,00,06,01,02,0,0,02,0,1, 12,07,03,00,04,00,22,1,1,01,4,0, 00,06,03,00,06,01,02,0,0,01,0,2);
                _MA3("midi.strings6", "Pizz.Str",  5,7, 00,15,11,05,11,09,20,0,0,01,0,0, 00,14,07,08,07,02,00,0,0,01,0,0, 08,15,07,06,05,15,17,0,1,01,0,0, 00,12,06,05,06,15,00,0,0,01,0,0);
                _MA3("midi.strings7", "Harp",      3,6, 00,15,08,05,08,04,41,0,0,02,0,0, 00,11,08,07,09,04,33,0,0,05,0,0, 00,11,07,03,02,04,33,0,2,01,0,0, 00,15,04,02,02,01,04,1,0,01,0,2);
                _MA3("midi.strings8", "Timpani",   3,3, 00,15,08,04,03,03,04,1,1,01,0,2, 00,15,02,02,02,15,33,1,0,00,7,2, 00,15,07,03,03,00,28,1,2,01,0,2, 08,15,04,03,03,15,00,1,0,00,0,2);

                _MA3("midi.ensemble1", "Strings1",  7,2, 07,05,10,00,06,00,11,0,2,01,1,1, 04,12,06,00,06,01,24,0,0,02,0,0, 00,06,06,00,06,01,07,0,2,01,5,0, 12,06,05,00,06,01,05,0,0,02,3,1);
                _MA3("midi.ensemble2", "Strings2",  7,3, 09,06,10,00,05,00,00,0,0,01,3,0, 02,12,06,00,05,01,23,0,0,01,0,1, 01,05,06,00,06,00,00,0,2,01,5,0, 27,05,05,00,06,01,07,0,0,01,7,0);
                _MA3("midi.ensemble3", "Syn.Str1",  5,0, 00,09,08,00,02,01,27,0,0,01,6,0, 00,07,15,00,05,00,07,0,0,01,3,0, 04,09,11,00,02,00,20,1,0,01,3,0, 00,06,15,00,04,00,00,0,1,01,7,0);
                _MA3("midi.ensemble4", "Syn.Str2",  5,5, 09,09,08,00,02,01,19,0,0,01,0,0, 00,06,06,00,05,00,06,0,0,01,0,0, 27,08,08,00,02,00,10,0,0,01,0,0, 00,05,07,00,04,03,00,0,1,01,0,0);
                _MA3("midi.ensemble5", "ChoirAah",  5,5, 07,12,00,00,00,15,19,0,0,06,0,0, 00,06,03,00,05,06,23,0,2,04,0,0, 08,07,15,00,03,00,30,0,1,01,0,0, 00,05,15,00,05,00,00,0,0,02,0,0);
                _MA3("midi.ensemble6", "VoiceOoh",  5,7, 07,12,00,00,00,15,20,1,0,05,0,1, 00,07,03,00,05,06,20,0,2,04,0,0, 10,07,07,00,04,03,26,0,2,01,0,0, 00,09,01,00,05,00,00,0,0,01,0,0);
                _MA3("midi.ensemble7", "SynVoice",  5,0, 00,10,00,00,04,15,22,0,0,01,0,0, 00,07,15,00,05,01,08,0,2,01,0,0, 09,09,15,00,04,00,26,0,1,01,0,0, 00,07,15,00,05,00,00,0,2,03,0,0);
                _MA3("midi.ensemble8", "Orch.Hit",  7,5, 08,15,04,04,06,06,00,1,0,04,0,1, 06,12,05,03,03,01,00,0,0,01,0,0, 06,12,07,06,06,00,00,0,0,01,0,0, 06,11,07,07,06,00,00,0,0,00,0,0);

                _MA3("midi.brass1",    "Trumpet",   3,6, 01,08,08,00,05,01,20,0,0,01,0,0, 00,10,08,00,05,04,23,0,2,03,0,0, 00,07,07,00,06,01,26,0,2,01,0,0, 00,09,15,00,08,00,06,0,0,01,0,0);
                _MA3("midi.brass2",    "Trombone",  3,7, 08,07,06,00,07,01,28,0,0,01,0,0, 00,09,08,00,05,04,15,0,2,01,0,0, 01,06,07,00,07,01,26,0,2,01,0,0, 00,08,15,00,08,00,05,0,0,01,0,0);
                _MA3("midi.brass3",    "Tuba",      3,7, 08,06,05,00,06,02,34,0,0,01,0,0, 01,12,08,00,11,04,24,0,2,01,0,0, 01,09,07,00,09,03,17,0,2,01,0,0, 00,07,15,00,08,00,00,0,0,02,0,0);
                _MA3("midi.brass4",    "Mute.Trp",  5,0, 10,07,00,00,07,05,26,0,0,03,0,0, 02,09,13,00,09,04,00,0,0,00,0,0, 17,07,09,00,06,01,19,0,0,05,0,0, 02,08,07,00,09,02,00,0,0,01,0,0);
                _MA3("midi.brass5",    "Fr.Horn",   5,0, 08,07,09,00,00,00,16,0,3,01,2,0, 00,09,14,00,07,00,01,0,2,01,7,0, 00,06,09,00,02,01,22,0,2,01,6,1, 08,10,14,00,07,00,01,0,2,01,3,0);
                _MA3("midi.brass6",    "BrasSect",  5,6, 00,08,06,00,02,01,22,0,0,01,7,2, 00,09,15,00,08,00,08,0,0,01,7,2, 12,07,07,00,05,01,22,0,0,01,0,2, 00,09,08,00,08,00,07,0,0,01,0,2);
                _MA3("midi.brass7",    "SynBras1",  5,6, 00,07,06,00,08,02,16,0,0,01,7,2, 00,09,15,00,10,00,10,0,0,01,7,2, 00,07,06,00,08,02,16,0,0,01,0,2, 00,09,08,00,10,00,10,0,0,01,0,2);
                _MA3("midi.brass8",    "SynBras2",  3,6, 00,06,03,00,04,01,28,0,0,01,0,0, 01,09,07,00,05,07,39,0,0,06,0,0, 08,07,05,03,03,11,35,0,0,01,0,0, 00,15,15,00,07,00,04,0,0,01,0,0);

                _MA3("midi.reed1",     "SprnoSax",  3,0, 00,15,09,06,06,03,29,1,0,03,0,0, 00,08,02,00,06,00,26,0,0,01,0,0, 01,08,05,00,00,00,12,0,1,01,0,0, 00,08,06,00,08,01,03,0,0,01,0,0);
                _MA3("midi.reed2",     "Alto Sax",  5,4, 01,09,03,00,00,00,10,1,2,01,0,2, 00,08,02,00,09,00,09,0,0,01,0,2, 09,09,03,00,00,00,13,1,2,01,0,2, 01,09,02,00,09,00,21,0,0,01,0,2);
                _MA3("midi.reed3",     "TenorSax",  5,3, 01,07,03,00,00,00,05,1,2,01,0,2, 08,07,02,00,09,00,15,0,0,01,0,2, 09,07,03,00,00,00,08,1,2,01,0,2, 00,07,02,00,09,00,13,0,2,01,0,2);
                _MA3("midi.reed4",     "Bari.Sax",  5,6, 00,07,03,00,05,00,18,1,2,01,0,0, 00,07,02,00,08,02,06,1,0,02,0,0, 02,07,05,00,01,00,14,1,1,02,0,0, 00,07,04,00,08,01,05,1,0,01,0,0);
                _MA3("midi.reed5",     "Oboe",      5,0, 05,10,00,00,04,00,30,0,2,01,0,0, 00,09,01,01,09,00,09,0,0,03,0,0, 00,11,00,00,04,02,24,0,0,01,0,1, 00,10,00,00,10,00,14,0,0,02,0,0);
                _MA3("midi.reed6",     "Eng.Horn",  5,0, 05,10,00,00,04,00,34,0,2,01,0,0, 00,09,01,01,09,01,11,0,0,03,0,0, 00,11,00,00,04,02,24,0,0,01,0,1, 00,10,00,01,10,01,11,0,0,02,0,0);
                _MA3("midi.reed7",     "Bassoon",   5,0, 01,12,07,00,00,01,24,1,2,01,0,0, 09,07,01,00,08,01,00,1,2,03,0,0, 01,12,07,00,00,01,24,1,2,01,0,0, 09,07,01,00,08,01,03,1,0,03,1,0);
                _MA3("midi.reed8",     "Clarinet",  5,7, 00,07,02,00,01,01,37,1,2,02,0,0, 00,08,02,00,08,01,03,0,0,01,0,0, 00,05,02,00,01,01,26,1,1,04,0,0, 00,07,02,00,08,01,03,0,0,01,0,0);
                
                _MA3("midi.pipe1",     "Piccolo",   5,7, 02,10,12,00,07,01,12,0,0,05,0,0, 00,09,07,00,08,15,39,0,0,01,0,0, 01,08,05,00,07,01,30,0,0,01,0,0, 00,08,05,00,10,00,06,0,0,01,0,0);
                _MA3("midi.pipe2",     "Flute",     5,7, 00,13,10,00,01,01,07,0,0,03,0,0, 00,07,08,00,11,03,37,0,0,03,0,0, 00,14,08,00,09,00,39,0,0,01,0,1, 16,06,05,00,10,00,01,0,0,01,0,0);
                _MA3("midi.pipe3",     "Recorder",  5,7, 03,09,06,07,10,00,58,0,0,02,0,1, 08,08,05,00,10,00,04,0,0,01,0,0, 24,10,09,06,06,09,15,0,0,07,0,0, 00,08,05,00,10,00,36,0,0,01,0,1);
                _MA3("midi.pipe4",     "PanFlute",  5,7, 00,10,00,00,06,00,00,0,1,13,0,1, 03,11,10,01,10,00,35,0,0,10,0,1, 00,08,15,00,04,00,44,0,0,02,0,0, 00,08,00,00,09,00,05,0,0,01,0,1);
                _MA3("midi.pipe5",     "Bottle",    5,7, 10,12,12,00,07,01,12,1,0,05,0,0, 00,07,07,00,09,06,27,0,0,01,0,0, 08,07,08,00,08,03,11,0,3,02,0,1, 00,07,05,00,08,00,01,0,0,01,0,0);
                _MA3("midi.pipe6",     "Shakhchi",  5,7, 02,10,12,00,05,01,06,0,0,05,0,0, 08,06,07,00,09,05,23,0,0,01,0,1, 18,10,08,00,03,03,02,0,1,00,0,2, 00,06,05,00,09,00,01,0,0,01,0,0);
                _MA3("midi.pipe7",     "Whistle",   2,0, 00,06,10,00,06,00,05,0,0,01,0,0, 00,08,08,00,07,00,05,0,0,01,0,2, 17,06,10,09,07,00,44,0,0,01,0,2, 08,06,08,00,07,00,05,0,0,01,7,2);
                _MA3("midi.pipe8",     "Ocarina",   5,7, 03,08,06,07,08,00,60,0,1,02,0,0, 00,08,05,00,09,00,00,0,0,01,0,0, 24,10,09,06,06,09,15,0,0,01,0,0, 08,07,05,00,10,00,26,0,0,01,0,0);
                
                _MA3("midi.lead1", "SquareLd",  5,0, 08,15,15,00,07,04,46,0,3,01,0,0, 08,10,15,00,10,00,03,0,2,01,0,0, 08,15,15,00,02,03,38,0,3,02,0,0, 08,10,15,00,10,00,03,0,2,01,0,0);
                _MA3("midi.lead2", "Saw.Lead",  5,7, 00,15,00,00,07,00,26,0,0,01,7,2, 08,13,15,00,07,00,10,0,0,01,7,2, 20,15,15,00,03,00,20,0,1,01,0,2, 08,14,15,00,08,00,10,0,0,01,0,2);
                _MA3("midi.lead3", "CaliopLd",  5,7, 08,12,04,00,07,08,00,0,0,04,0,1, 08,12,06,00,06,07,20,0,1,04,0,2, 00,08,06,00,05,05,03,0,1,02,0,0, 16,06,04,00,08,01,02,0,0,01,0,1);
                _MA3("midi.lead4", "ChiffLd",   5,0, 08,07,07,00,02,06,04,0,1,01,4,0, 08,15,06,00,08,00,11,0,0,01,0,0, 08,07,07,00,02,06,03,0,1,01,1,0, 08,15,06,00,08,00,11,0,0,01,2,0);
                _MA3("midi.lead5", "CharanLd",  5,0, 01,09,02,00,06,02,08,0,2,01,4,2, 12,09,01,00,08,02,14,0,0,02,0,0, 01,09,02,00,06,02,10,0,2,01,1,2, 12,09,01,00,08,02,14,0,0,02,1,0);
                _MA3("midi.lead6", "Voice Ld",  5,0, 06,04,00,00,00,15,13,0,0,07,0,0, 00,07,03,00,08,06,19,0,0,02,0,0, 08,07,15,00,09,00,28,0,1,01,0,1, 00,07,15,00,08,00,02,0,0,01,0,0);
                _MA3("midi.lead7", "Fifth Ld",  7,0, 24,12,01,01,08,01,21,0,0,01,0,2, 00,12,01,01,06,02,15,0,0,01,1,2, 00,12,00,00,08,00,26,0,0,02,4,2, 16,12,01,01,08,01,06,0,0,03,1,2);
                _MA3("midi.lead8", "Bass &Ld",  5,0, 01,11,02,00,03,00,22,0,0,01,0,0, 16,10,02,00,09,00,17,0,0,01,0,0, 00,12,03,04,04,05,11,0,2,01,0,0, 00,13,03,00,09,06,12,0,0,01,0,0);

                _MA3("midi.pad1",  "NewAgePd",  5,5, 01,15,15,03,03,00,38,1,0,07,0,2, 00,15,07,04,04,00,11,0,2,05,0,2, 01,06,01,00,01,00,24,0,2,01,7,0, 00,08,01,00,05,01,00,0,2,01,0,0);
                _MA3("midi.pad2",  "Warm Pad",  5,7, 00,10,00,00,05,00,40,0,0,01,1,0, 00,03,00,00,04,00,02,0,0,01,0,0, 00,10,00,00,03,00,47,0,0,01,3,0, 00,03,00,00,04,00,02,0,0,01,2,0);
                _MA3("midi.pad3",  "PolySyPd",  5,0, 03,06,05,00,04,01,34,0,0,01,7,0, 00,10,05,00,06,01,02,0,2,02,6,0, 27,06,03,00,03,01,34,0,0,01,0,0, 08,09,03,00,05,00,00,0,2,01,2,0);
                _MA3("midi.pad4",  "ChoirPad",  5,2, 08,10,00,00,00,15,33,0,0,01,7,2, 00,04,03,00,07,03,22,0,2,08,0,0, 00,07,03,00,03,00,33,0,1,01,0,0, 00,06,15,00,05,00,00,0,0,02,3,0);
                _MA3("midi.pad5",  "BowedPad",  5,4, 00,02,01,00,03,04,42,0,2,07,0,0, 00,06,02,00,05,03,00,0,0,01,0,0, 00,02,01,00,03,04,42,0,2,07,1,0, 00,06,02,00,05,03,00,0,0,01,2,0);
                _MA3("midi.pad6",  "MetalPad",  5,6, 10,15,02,00,03,00,23,1,2,01,0,0, 00,05,06,00,04,00,03,0,2,01,5,0, 00,15,06,00,03,03,07,1,1,01,0,0, 00,05,07,00,04,00,04,0,1,01,1,0);
                _MA3("midi.pad7",  "Halo Pad",  5,6, 00,04,01,00,03,01,37,0,0,01,5,2, 00,06,02,00,05,00,00,0,2,01,7,0, 00,12,05,00,04,01,30,0,0,01,3,2, 00,08,02,00,05,00,00,0,2,01,0,0);
                _MA3("midi.pad8",  "SweepPad",  5,0, 00,04,08,00,03,00,30,0,0,01,5,0, 00,03,08,00,04,00,00,0,2,01,3,0, 00,03,01,00,04,00,34,0,2,02,2,0, 00,07,02,00,05,00,00,0,2,01,7,0);

                _MA3("midi.fx1", "Rain",      5,1, 00,15,08,00,06,08,02,1,3,10,0,2, 16,08,05,01,02,00,00,1,0,01,0,3, 00,15,08,00,06,08,02,1,3,10,2,2, 16,08,05,01,02,00,00,1,0,01,3,2);
                _MA3("midi.fx2", "SoundTrk",  5,3, 00,06,03,00,03,03,18,0,2,03,0,0, 00,04,02,00,04,00,09,0,1,03,0,0, 16,06,03,00,03,03,16,0,0,01,0,0, 00,04,01,00,04,01,03,0,2,01,0,0);
                _MA3("midi.fx3", "Crystal",   5,5, 00,15,08,01,05,04,20,0,1,06,0,2, 00,12,02,02,04,07,09,0,0,01,0,2, 00,15,08,01,05,04,20,0,1,14,2,2, 00,12,02,02,04,07,09,0,0,01,2,2);
                _MA3("midi.fx4", "Atmosphr",  5,4, 20,06,03,02,04,15,21,0,1,01,0,0, 00,15,03,00,04,03,00,0,0,01,0,0, 20,12,03,00,04,15,16,0,1,02,3,0, 00,09,06,05,04,03,12,0,0,02,7,0);
                _MA3("midi.fx5", "Bright",    2,0, 11,15,01,01,04,05,07,0,2,01,3,0, 09,15,02,05,04,15,07,0,0,01,3,0, 11,15,01,01,04,05,07,0,2,01,1,0, 09,15,02,05,04,15,07,0,0,01,1,0);
                _MA3("midi.fx6", "Goblins",   5,4, 13,01,01,00,01,02,18,0,2,03,7,2, 00,02,01,00,04,01,09,0,2,03,0,0, 00,02,01,00,02,00,20,0,0,01,0,2, 00,03,01,00,03,00,03,0,2,01,4,2);
                _MA3("midi.fx7", "Echoes",    5,0, 00,04,03,00,00,05,34,0,0,02,0,0, 00,10,02,00,12,00,14,0,2,01,0,0, 16,03,03,00,02,02,35,0,0,01,0,1, 00,10,15,00,03,00,00,0,0,01,0,0);
                _MA3("midi.fx8", "Sci-Fi",    5,2, 01,05,03,00,03,08,25,0,1,02,0,0, 04,06,01,01,04,02,06,0,0,01,7,0, 01,05,03,00,03,08,25,0,1,02,3,0, 04,06,01,01,04,02,06,0,0,01,3,0);

                _MA3("midi.world1",  "Sitar",     5,3, 00,13,02,03,02,05,10,0,2,02,0,2, 09,15,02,06,04,15,08,0,0,07,0,2, 08,13,02,03,02,05,03,0,2,01,1,2, 17,15,02,06,04,15,08,0,0,04,1,2);
                _MA3("midi.world2",  "Banjo",     5,0, 04,13,03,02,01,01,10,1,0,01,0,0, 00,13,03,03,05,14,00,1,2,03,0,0, 01,15,07,04,02,01,15,0,0,06,0,0, 01,15,07,08,08,14,00,0,2,01,0,0);
                _MA3("midi.world3",  "Shamisen",  3,4, 08,15,01,01,03,02,26,0,0,01,0,2, 00,15,10,05,06,07,20,0,0,03,0,2, 16,15,08,03,03,03,24,0,0,05,0,2, 08,15,04,04,04,15,01,1,0,03,0,2);
                _MA3("midi.world4",  "Koto",      3,6, 08,15,07,05,05,02,20,0,2,03,0,0, 16,15,09,08,08,04,21,0,2,05,0,0, 00,15,02,02,03,15,42,0,0,01,0,0, 00,15,02,02,02,15,03,1,0,01,0,2);
                _MA3("midi.world5",  "Kalimba",   5,6, 04,15,10,05,06,10,08,0,1,04,3,2, 00,12,02,03,05,00,08,0,0,01,0,2, 00,15,08,05,06,10,14,0,1,05,2,2, 00,12,02,04,04,00,08,1,0,01,4,2);
                _MA3("midi.world6",  "Bagpipe",   5,1, 09,11,09,00,12,01,16,0,2,01,0,2, 08,07,15,00,13,00,11,0,0,03,1,2, 00,10,09,00,09,03,00,0,0,01,0,2, 01,08,06,00,13,00,10,0,0,04,0,2);
                _MA3("midi.world7",  "Fiddle",    5,2, 01,08,09,00,03,01,07,1,2,01,0,0, 00,06,06,00,07,02,00,0,0,01,0,2, 04,12,06,07,07,00,09,1,1,02,0,0, 00,09,03,07,08,14,25,0,2,01,0,0);
                _MA3("midi.world8",  "Shanai",    5,0, 05,10,00,00,04,00,16,0,2,01,0,0, 00,09,01,01,09,00,13,0,0,06,0,0, 00,11,00,00,04,02,24,0,0,01,0,1, 00,10,00,00,10,00,15,0,0,02,0,0);

                _MA3("midi.percus1",  "TnklBell",  5,3, 00,15,06,03,04,05,16,0,1,14,0,2, 00,12,06,07,06,14,11,0,2,02,0,2, 01,12,06,02,02,05,30,0,0,07,7,2, 00,15,05,04,05,13,01,0,0,06,0,2);
                _MA3("midi.percus2",  "Agogo",     5,1, 00,14,10,04,04,02,23,0,0,07,0,2, 00,15,07,06,06,01,08,0,0,05,0,2, 00,14,09,06,04,02,33,0,0,10,7,2, 00,15,07,06,06,07,04,0,0,02,0,2);
                _MA3("midi.percus3",  "SteelDrm",  7,0, 16,04,04,04,05,02,00,1,0,02,0,2, 00,06,06,03,04,00,22,0,0,02,0,2, 00,14,04,04,04,02,00,1,0,02,0,2, 02,15,04,04,04,02,00,1,0,00,3,0);
                _MA3("midi.percus4",  "WoodBlok",  5,5, 00,15,10,09,09,02,33,1,2,05,1,2, 00,15,10,07,07,02,00,1,0,02,3,2, 00,15,10,10,08,02,28,0,0,10,3,2, 00,15,10,07,07,02,00,0,0,02,0,2);
                _MA3("midi.percus5",  "TaikoDrm",  4,7, 04,15,11,04,03,05,24,0,0,00,0,0, 00,15,12,10,03,01,19,0,0,04,0,0, 00,15,12,04,03,04,11,0,0,03,0,0, 00,15,14,05,05,00,00,0,0,00,0,0);
                _MA3("midi.percus6",  "MelodTom",  5,7, 04,15,09,08,08,05,14,0,2,01,0,2, 00,15,03,05,05,14,00,0,0,00,0,2, 01,15,03,04,04,05,12,1,2,00,0,2, 00,15,04,04,04,14,00,1,0,00,0,2);
                _MA3("midi.percus7",  "Syn.Drum",  5,2, 12,14,11,08,10,02,00,1,0,00,7,0, 00,12,03,00,04,15,00,1,0,00,0,0, 24,15,04,00,04,07,00,0,0,00,0,2, 22,15,07,00,07,15,00,0,0,00,0,2);
                _MA3("midi.percus8",  "RevCymbl",  5,7, 00,04,15,00,00,00,00,0,0,14,0,2, 08,02,15,15,15,15,06,0,0,09,0,1, 00,04,15,00,00,00,00,0,0,14,0,2, 03,02,15,15,15,15,06,0,0,14,0,2);

                _MA3("midi.se1",  "FretNoiz",  5,7, 08,15,08,04,06,02,00,0,2,06,0,2, 02,08,06,04,10,08,15,0,2,03,0,2, 10,15,06,02,06,02,00,0,0,06,0,2, 02,08,08,08,10,08,00,0,2,06,0,2);
                _MA3("midi.se2",  "BrthNoiz",  5,7, 02,10,12,00,05,01,14,0,0,05,0,0, 00,07,08,08,09,15,05,0,0,01,0,0, 00,10,12,00,05,01,09,0,0,08,0,0, 00,08,08,07,09,15,12,0,0,01,0,0);
                _MA3("midi.se3",  "Seashore",  3,7, 00,15,15,00,00,00,04,0,0,03,0,0, 04,15,02,02,02,15,21,0,0,00,0,0, 01,15,00,04,04,15,12,0,0,00,0,0, 00,01,02,04,04,15,00,0,0,00,0,0);
                _MA3("midi.se4",  "Tweet",     5,0, 00,03,07,03,03,10,21,1,2,05,0,2, 00,05,06,07,07,03,00,1,1,10,0,2, 00,03,07,03,03,10,21,1,2,05,0,2, 00,05,06,04,04,03,00,1,1,10,7,3);
                _MA3("midi.se5",  "Telphone",  5,5, 02,11,02,00,00,06,28,1,1,05,0,2, 00,15,04,03,03,01,03,1,0,04,0,2, 02,11,02,00,00,06,28,1,1,05,0,2, 00,15,04,03,03,01,15,1,0,04,7,2);
                _MA3("midi.se6",  "Helicptr",  5,5, 14,15,06,00,00,00,01,0,0,15,0,2, 00,02,00,00,05,00,04,0,0,00,0,2, 14,15,00,00,00,00,24,0,0,00,0,2, 00,02,00,00,05,00,11,0,0,00,0,2);
                _MA3("midi.se7",  "Applause",  5,7, 24,15,12,00,01,00,00,0,0,00,0,0, 00,04,02,00,05,00,08,0,0,00,0,0, 24,06,00,00,01,15,00,0,0,09,0,1, 00,03,00,07,07,15,07,0,0,03,0,0);
                _MA3("midi.se8",  "Gunshot",   5,7, 00,15,03,00,00,15,11,0,0,05,0,0, 06,15,06,08,08,11,04,0,0,15,0,0, 01,15,02,00,00,15,02,0,0,05,0,0, 06,15,06,08,08,11,26,0,0,05,0,0);

                // 128 voices from GM
            }
            
            if (includeFlag & INCLUDE_MIDIDRUM) {
                _categoly("midi.drum");
                _MA3("midi.drum24",  "Seq Click H",      5,6, 04,15,00,15,15,00,56,0,0,06,0,2, 00,15,08,15,15,12,00,1,0,05,0,2, 06,15,11,15,15,11,39,0,0,10,0,2, 00,14,11,15,15,15,01,1,0,05,0,2);
                _MA3("midi.drum25",  "Brush Tap",        5,7, 00,15,08,00,00,03,00,0,0,05,0,2, 00,09,08,08,11,11,00,0,2,00,0,2, 06,15,08,06,14,03,36,0,0,12,0,2, 06,12,13,08,08,00,44,0,2,01,0,2);
                _MA3("midi.drum26",  "Brush Swirl",      5,7, 18,15,05,00,06,00,00,0,0,00,0,2, 03,09,08,00,10,06,00,0,0,00,0,3, 16,15,00,00,15,15,00,0,0,00,0,0, 00,03,06,03,15,06,21,0,0,01,0,3);
                _MA3("midi.drum27",  "Brush Slap",       5,7, 00,15,00,00,00,00,00,0,0,04,0,2, 13,12,08,06,12,09,00,0,0,14,0,2, 07,11,10,04,07,13,09,0,0,00,0,2, 00,08,08,11,11,13,00,0,2,00,0,2);
                _MA3("midi.drum28",  "Brush Tap Swirl",  5,7, 18,15,05,00,03,00,00,0,0,05,5,2, 03,09,06,00,10,06,00,0,0,03,0,3, 16,15,00,00,15,15,00,0,0,09,0,0, 20,03,06,03,15,06,16,0,0,03,0,3);
                _MA3("midi.drum29",  "Snare Roll",       5,7, 02,07,00,00,02,00,13,0,0,08,3,1, 26,15,05,00,09,03,00,0,0,00,6,0, 00,15,10,08,04,03,08,0,0,03,0,2, 00,14,06,07,07,05,00,0,0,00,0,2);
                _MA3("midi.drum30",  "Castanet",         5,6, 01,15,07,05,09,15,02,0,0,07,0,2, 06,10,08,05,15,15,00,1,0,05,0,2, 05,15,05,06,05,00,39,0,0,02,0,2, 00,12,10,09,09,10,20,0,0,05,0,2);
                _MA3("midi.drum31",  "Snare L",          5,7, 24,15,12,00,00,01,09,0,0,00,0,2, 00,15,07,07,07,03,00,0,0,00,0,2, 00,14,11,07,05,15,48,0,0,07,0,2, 00,15,10,06,00,00,28,0,0,00,0,2);
                _MA3("midi.drum32",  "Sticks",           6,7, 20,15,09,12,08,09,00,0,1,15,0,2, 03,13,10,02,08,11,00,0,0,10,0,2, 03,13,08,02,05,05,21,0,1,11,0,2, 08,12,11,09,07,11,00,0,0,13,0,2);
                _MA3("midi.drum33",  "Bass Drum L",      5,1, 04,15,10,11,06,15,00,0,0,02,0,2, 00,15,06,06,05,07,16,0,0,02,0,2, 01,11,09,07,04,07,13,0,0,05,0,2, 12,15,07,08,05,07,00,0,0,01,0,2);
                _MA3("midi.drum34",  "Open Rim Shot",    5,7, 02,15,00,05,05,00,05,1,0,12,0,2, 00,15,07,07,07,07,00,1,0,11,0,2, 00,15,10,06,06,08,00,0,0,12,0,2, 02,15,07,07,07,07,00,0,0,07,0,2);
                _MA3("midi.drum35",  "Bass Drum M",      5,6, 06,15,13,07,07,15,00,0,0,13,0,2, 00,15,07,07,04,07,00,0,0,02,0,2, 10,15,09,07,06,07,19,0,0,03,0,2, 12,15,08,10,06,11,00,0,0,02,0,2);
                _MA3("midi.drum36",  "Bass Drum H",      5,6, 06,15,13,07,06,15,14,0,0,13,0,2, 00,15,07,07,07,07,12,0,0,02,0,2, 02,15,09,07,04,07,09,0,0,02,0,2, 12,15,07,08,06,07,00,0,0,01,0,2);
                _MA3("midi.drum37",  "Closed Rim Shot",  5,6, 01,15,02,00,08,05,08,0,0,10,0,2, 06,13,09,03,03,15,00,1,0,07,0,2, 18,11,00,00,09,13,00,0,0,09,0,2, 18,13,09,11,09,11,00,0,0,00,0,2);
                _MA3("midi.drum38",  "Snare M",          5,7, 13,15,00,05,07,00,07,1,0,12,0,2, 00,15,07,10,09,07,00,1,0,08,0,2, 00,15,05,07,08,06,00,0,0,12,0,2, 02,15,07,07,06,07,01,0,0,07,0,2);
                _MA3("midi.drum39",  "Hand Clap",        5,7, 00,15,04,04,06,00,02,0,0,00,0,2, 05,15,06,09,04,01,00,0,0,00,0,2, 27,15,08,00,15,01,00,0,2,13,1,2, 02,15,10,09,08,05,00,0,2,15,0,2);
                _MA3("midi.drum40",  "Snare H",          5,7, 02,15,01,00,07,00,10,0,0,12,0,2, 00,15,07,11,07,13,00,0,0,11,0,0, 28,15,10,06,05,08,09,0,0,09,0,2, 18,15,08,10,06,01,00,0,0,07,0,2);
                _MA3("midi.drum41",  "Floor Tom L",      5,4, 06,13,10,06,06,15,09,0,0,12,0,2, 00,15,10,07,11,07,00,0,0,05,0,2, 01,15,03,05,05,15,21,0,0,00,0,2, 00,15,06,05,06,07,00,0,0,05,0,2);
                _MA3("midi.drum42",  "Hi-Hat Closed",    5,7, 00,15,00,00,00,03,02,0,0,15,0,2, 00,11,08,12,08,11,00,0,2,00,0,2, 14,15,03,03,13,03,12,0,0,00,0,2, 11,11,10,08,11,00,00,0,2,13,0,2);
                _MA3("midi.drum43",  "Floor Tom H",      5,0, 06,12,05,06,06,15,00,0,0,12,0,2, 19,15,09,07,11,07,00,0,0,11,0,2, 01,15,04,04,05,15,26,0,0,00,0,2, 00,15,06,04,06,05,00,0,0,05,0,2);
                _MA3("midi.drum44",  "Hi-Hat Pedal",     5,7, 00,15,00,00,00,03,00,0,0,12,0,2, 00,07,08,08,08,11,15,0,2,00,0,2, 20,15,03,05,14,03,08,0,0,06,0,2, 11,08,03,08,08,00,00,0,2,05,0,2);
                _MA3("midi.drum45",  "Low Tom",          5,0, 06,12,05,06,06,15,01,0,0,10,0,2, 18,15,08,07,09,07,00,0,0,01,0,2, 01,15,03,05,05,15,17,0,0,00,0,2, 00,15,06,05,05,07,00,0,0,05,0,2);
                _MA3("midi.drum46",  "Hi-Hat Open",      5,7, 00,15,00,00,00,03,02,0,0,15,0,2, 00,11,07,06,08,11,01,0,2,00,0,2, 14,15,03,03,13,03,12,0,0,00,0,2, 11,11,09,05,11,00,09,0,2,13,0,2);
                _MA3("midi.drum47",  "Mid Tom L",        5,0, 06,12,05,06,06,15,00,0,0,10,0,2, 18,15,09,07,10,06,00,0,0,08,0,2, 00,15,04,02,05,15,28,0,0,00,0,2, 00,15,06,05,07,07,02,0,0,04,0,2);
                _MA3("midi.drum48",  "Mid Tom H",        5,0, 06,12,04,06,05,15,09,0,0,10,0,2, 19,15,09,07,07,07,00,0,0,01,0,2, 01,15,04,05,05,15,39,0,0,01,0,2, 00,15,06,06,07,07,03,0,0,04,0,2);
                _MA3("midi.drum49",  "Crash Cymbal 1",   5,3, 16,15,09,00,06,00,14,0,0,13,0,2, 18,09,03,04,05,02,03,0,0,11,0,0, 00,11,03,00,04,00,00,0,0,15,0,2, 06,12,04,05,05,07,13,0,0,15,0,2);
                _MA3("midi.drum50",  "High Tom",         5,0, 06,12,04,06,07,15,04,0,0,08,0,2, 19,15,10,07,08,07,00,0,0,06,0,2, 05,15,05,05,06,15,50,0,0,01,0,2, 00,12,06,05,06,07,00,0,0,05,0,2);
                _MA3("midi.drum51",  "Ride Cymbal 1",    5,7, 00,15,07,00,00,04,09,0,2,15,0,2, 14,14,04,04,04,14,03,0,0,09,0,2, 00,15,00,00,00,04,08,0,2,14,0,2, 11,15,04,02,05,13,16,0,0,14,0,2);
                _MA3("midi.drum52",  "Chinese Cymbal",   5,7, 00,15,03,07,02,01,31,0,0,00,0,2, 30,14,03,05,02,00,00,1,0,05,0,0, 00,15,01,08,03,06,15,0,0,02,0,2, 06,09,02,05,03,00,09,0,0,00,0,2);
                _MA3("midi.drum53",  "Ride Cymbal Cup",  5,7, 19,15,07,00,00,04,22,0,2,15,0,2, 12,15,05,04,04,14,00,0,0,15,0,2, 19,15,00,00,00,04,12,0,2,12,0,2, 11,15,05,02,05,13,27,0,0,15,0,2);
                _MA3("midi.drum54",  "Tambourine",       5,6, 00,08,07,04,01,02,10,0,0,11,0,2, 13,14,07,12,11,05,00,0,0,05,0,2, 08,08,07,05,02,02,00,0,0,11,0,2, 08,13,07,12,07,06,00,0,0,15,0,2);
                _MA3("midi.drum55",  "Splash Cymbal",    5,7, 25,12,06,03,03,00,00,0,0,08,0,0, 03,09,03,04,03,12,00,1,0,13,0,0, 13,05,00,03,03,00,01,0,0,05,0,0, 19,12,03,05,08,06,00,1,0,15,0,0);
                _MA3("midi.drum56",  "Cowbell",          5,4, 00,15,09,08,05,03,00,1,2,03,0,2, 00,10,07,05,03,03,00,1,0,00,0,2, 02,15,15,11,09,10,03,1,0,04,0,2, 00,15,12,05,05,03,32,1,0,01,0,2);
                _MA3("midi.drum57",  "Crash Cymbal 2",   5,7, 00,15,06,00,00,00,00,0,2,09,0,2, 00,09,04,02,02,00,00,1,0,00,0,0, 00,15,03,00,00,00,00,0,2,15,0,2, 02,09,06,03,04,03,08,0,0,07,4,2);
                _MA3("midi.drum58",  "Vibraslap",        5,7, 29,13,01,02,06,00,00,0,0,01,2,1, 08,15,06,05,09,02,00,0,0,00,6,0, 29,15,05,04,04,00,29,0,0,06,0,2, 30,15,06,05,04,00,00,0,0,00,0,2);
                _MA3("midi.drum59",  "Ride Cymbal 2",    5,7, 07,15,00,00,06,00,05,0,0,14,0,3, 14,14,05,05,04,11,16,0,0,15,0,2, 06,15,00,00,00,04,07,0,2,09,0,2, 11,15,05,03,05,14,00,0,0,09,0,2);
                _MA3("midi.drum60",  "Bongo H",          5,3, 00,15,05,12,12,00,00,0,0,06,0,2, 00,15,05,08,07,00,00,0,0,11,0,2, 00,15,05,12,12,00,02,0,0,15,0,2, 00,15,04,07,03,00,00,0,0,15,0,2);
                _MA3("midi.drum61",  "Bongo L",          5,3, 00,15,05,12,12,00,00,0,0,06,0,2, 00,15,05,08,07,00,07,0,0,11,0,2, 00,15,05,12,12,00,06,0,0,15,0,2, 00,15,05,08,03,00,00,0,0,15,0,2);
                _MA3("midi.drum62",  "Conga H Mute",     5,4, 00,14,14,10,10,01,06,0,0,00,2,2, 08,15,08,09,10,01,00,0,0,01,0,2, 00,15,00,00,00,00,44,0,0,00,0,2, 00,15,13,09,10,01,07,0,0,00,0,2);
                _MA3("midi.drum63",  "Conga H Open",     5,7, 03,10,10,05,08,12,38,0,0,10,0,2, 00,12,06,07,09,04,00,0,0,12,0,2, 00,15,10,06,08,08,00,0,0,12,0,2, 02,15,07,07,09,07,00,0,0,02,0,2);
                _MA3("midi.drum64",  "Conga L",          5,7, 03,10,10,05,08,12,31,0,0,10,0,2, 00,12,05,06,09,04,00,0,0,12,0,2, 00,15,10,06,08,08,00,0,0,12,0,2, 10,15,07,07,09,07,00,0,0,02,0,2);
                _MA3("midi.drum65",  "Timbale H",        3,6, 00,12,07,00,08,00,22,0,0,02,0,2, 00,15,09,06,08,00,49,0,0,07,0,2, 04,13,06,10,06,01,26,0,0,09,0,2, 06,15,09,06,08,00,06,0,0,09,0,2);
                _MA3("midi.drum66",  "Timbale L",        3,6, 00,12,07,00,08,00,22,0,0,02,0,2, 00,15,09,06,08,00,49,0,0,07,0,2, 04,13,06,10,06,01,23,0,0,12,0,2, 06,15,09,06,08,00,04,0,0,09,0,2);
                _MA3("midi.drum67",  "Agogo H",          5,5, 00,14,12,06,06,02,21,0,0,07,0,2, 00,15,08,06,06,01,10,0,0,02,0,2, 00,14,12,06,05,02,26,0,0,07,0,2, 00,15,08,06,06,01,10,0,0,02,0,2);
                _MA3("midi.drum68",  "Agogo L",          5,5, 00,14,12,06,06,02,21,0,0,07,0,2, 00,15,08,06,06,01,10,0,0,02,0,2, 00,14,12,05,05,02,13,0,0,07,0,2, 00,15,08,06,06,01,29,0,0,02,0,2);
                _MA3("midi.drum69",  "Cabasa",           3,7, 00,15,00,03,05,06,00,0,0,07,0,2, 00,14,08,04,04,10,24,1,0,15,0,2, 00,14,04,02,06,04,00,0,0,15,0,2, 06,08,11,09,04,00,02,0,0,15,0,2);
                _MA3("midi.drum70",  "Maracas",          4,7, 00,15,00,03,05,06,04,0,0,15,0,2, 00,08,08,04,04,06,03,1,0,15,0,2, 06,12,09,03,03,15,00,0,0,15,0,2, 12,07,10,10,03,08,00,0,0,15,0,2);
                _MA3("midi.drum71",  "Samba Whistle H",  5,5, 01,15,00,00,07,00,10,0,0,00,0,1, 00,12,01,00,12,01,43,0,0,14,0,1, 00,15,00,00,07,00,23,0,0,00,0,2, 00,08,01,00,12,00,06,0,0,15,0,1);
                _MA3("midi.drum72",  "Samba Whistle L",  5,5, 01,15,00,00,07,00,10,0,0,00,0,1, 00,12,01,00,12,01,43,0,0,14,0,1, 00,15,00,00,07,00,23,0,0,00,0,2, 00,08,01,00,12,00,06,0,0,15,0,1);
                _MA3("midi.drum73",  "Guiro Short",      3,7, 01,12,02,06,00,00,12,0,0,01,0,0, 04,12,03,06,10,00,00,0,0,03,0,0, 11,13,01,00,06,00,32,0,0,14,5,0, 07,15,07,12,15,06,00,1,0,00,1,0);
                _MA3("midi.drum74",  "Guiro Long",       3,7, 01,12,02,06,00,00,14,0,0,01,0,0, 04,12,03,06,10,00,00,0,0,03,0,0, 11,15,00,00,06,00,32,0,0,13,0,0, 07,09,06,14,12,06,05,0,0,00,2,0);
                _MA3("midi.drum75",  "Claves",           4,0, 00,13,07,06,06,10,11,0,0,07,0,2, 00,15,14,11,03,10,13,0,0,07,0,2, 00,15,11,08,00,12,00,1,0,00,0,2, 08,15,03,08,08,00,00,1,0,15,0,2);
                _MA3("midi.drum76",  "Wood Block H",     4,0, 00,14,06,06,06,10,63,0,0,15,0,2, 00,15,06,06,05,11,00,0,0,06,0,2, 02,13,12,00,10,12,00,1,0,07,0,2, 08,15,07,12,07,04,00,1,0,03,0,2);
                _MA3("midi.drum77",  "Wood Block L",     4,0, 00,14,06,06,06,10,63,0,0,15,0,2, 00,15,06,06,05,11,00,0,0,06,0,2, 02,15,10,00,10,12,16,1,0,10,0,2, 16,12,07,12,12,04,00,1,0,03,0,2);
                _MA3("midi.drum78",  "Cuica Mute",       5,0, 00,13,10,12,13,11,15,0,2,00,0,2, 00,06,00,06,05,00,06,0,0,06,0,2, 00,15,08,08,12,00,14,0,0,00,0,2, 00,06,00,07,07,00,02,0,0,06,0,2);
                _MA3("midi.drum79",  "Cuica Open",       5,1, 16,09,08,15,08,15,12,0,0,00,0,0, 08,07,06,08,08,04,00,0,0,14,0,0, 00,15,07,00,00,00,36,0,0,02,0,2, 00,13,07,11,11,12,18,0,0,11,0,2);
                _MA3("midi.drum80",  "Triangle Mute",    7,0, 22,15,12,08,04,00,05,0,0,09,0,2, 04,11,05,09,05,06,00,0,0,15,0,2, 06,15,11,12,07,03,07,0,0,14,0,2, 00,14,13,08,11,15,06,0,0,06,0,2);
                _MA3("midi.drum81",  "Triangle Open",    7,0, 22,15,05,05,04,00,03,0,0,09,0,2, 04,11,05,09,05,06,00,0,0,15,0,2, 06,15,09,09,07,03,08,0,0,14,0,2, 00,14,05,05,11,15,03,0,0,06,0,2);
                _MA3("midi.drum82",  "Shaker",           4,7, 00,09,00,03,05,06,00,0,0,07,0,2, 00,09,06,04,04,06,04,1,0,03,0,2, 00,14,04,02,06,04,01,0,0,12,0,2, 06,09,10,09,04,08,02,0,0,15,0,2);
                _MA3("midi.drum83",  "Jingle Bells",     5,0, 02,07,06,00,00,00,20,0,0,06,0,2, 03,06,11,05,05,04,00,0,0,02,0,2, 02,03,00,00,02,00,07,1,0,06,0,2, 02,07,05,02,06,15,00,0,0,09,0,2);
                _MA3("midi.drum84",  "Bell Tree",        5,7, 19,12,04,02,00,00,24,0,0,03,0,2, 02,04,03,05,05,04,00,0,0,02,0,3, 02,03,00,00,02,00,07,1,0,07,0,3, 13,05,04,02,06,06,02,0,0,03,0,3);

                // 60 voices from GM drum
            }
        }
        
        
        
        
    // internal functions
    //--------------------------------------------------
        // create new voice
        private function _new(key:String, name:String, voice:SiONVoice) : void {
            voice.name = name;
            _categolyList.push(voice);
            this[key] = voice;
        }
        
        
        // create new OPN voice
        private function _OPN(key:String, name:String, ...args) : void {
            var voice:SiONVoice = new SiONVoice();
            voice.paramOPN = args;
            voice.name = name;
            _categolyList.push(voice);
            this[key] = voice;
        }
        
        
        // create new MA3 voice
        private function _MA3(key:String, name:String, ...args) : void {
            var voice:SiONVoice = new SiONVoice();
            voice.paramMA3 = args;
            voice.name = name;
            _categolyList.push(voice);
            this[key] = voice;
        }
        
        
        // register categoly
        private var _categolyList:Array;
        private function _categoly(key:String) : void {
            _categolyList = [];
            _categolyList["name"] = key;
            categolies.push(_categolyList);
            this[key] = _categolyList;
        }
    }
}


